"""
    These are models relative facebook page.

    Note:
        Some field which not common used has not include.
"""
import cattr
from typing import Dict, Optional, List
from attr import attrs, attrib

from .base import BaseModel
from .picture import CoverPhoto, ProfilePictureSource
from .._compat import str


@attrs
class PageCategory(BaseModel):
    """
    A class representing the page category info.

    Refer: https://developers.facebook.com/docs/graph-api/reference/page-category/
    """

    id = attrib(default=None, type=Optional[str])
    api_enum = attrib(default=None, type=Optional[str], repr=False)
    fb_page_categories = attrib(default=None, type=Optional[List], repr=False)
    name = attrib(default=None, type=Optional[str])

    def __attrs_post_init__(self):
        """
        Because field for fb_page_categories is a list of categories which same structure as PageCategory.
        So need init by hands.
        """
        if isinstance(self.fb_page_categories, (list, tuple)):
            self.fb_page_categories = [cattr.structure(item, PageCategory) for item in self.fb_page_categories]
        else:
            pass


@attrs
class ContactAddress(BaseModel):
    """
    A class representing the mailing address info.

    Refer: https://developers.facebook.com/docs/graph-api/reference/mailing-address/
    """
    id = attrib(default=None, type=Optional[str])
    city = attrib(default=None, type=Optional[str])
    country = attrib(default=None, type=Optional[str])
    postal_code = attrib(default=None, type=Optional[str], repr=False)
    region = attrib(default=None, type=Optional[str], repr=False)
    street1 = attrib(default=None, type=Optional[str], repr=False)
    street2 = attrib(default=None, type=Optional[str], repr=False)


@attrs
class PageEngagement(BaseModel):
    """
    A class representing the page engagement info.

    Refer: https://developers.facebook.com/docs/graph-api/reference/engagement/
    """

    count = attrib(default=None, type=Optional[int])
    count_string = attrib(default=None, type=Optional[str], repr=False)
    count_string_with_like = attrib(default=None, type=Optional[str], repr=False)
    count_string_without_like = attrib(default=None, type=Optional[str], repr=False)
    social_sentence = attrib(default=None, type=Optional[str])
    social_sentence_with_like = attrib(default=None, type=Optional[str], repr=False)
    social_sentence_without_like = attrib(default=None, type=Optional[str], repr=False)


@attrs
class PageStartDate(BaseModel):
    """
    A class representing the page start date info.

    Refer: https://developers.facebook.com/docs/graph-api/reference/page-start-date/
    """
    day = attrib(default=None, type=Optional[int])
    month = attrib(default=None, type=Optional[int])
    year = attrib(default=None, type=Optional[int])


@attrs
class PageStartInfo(BaseModel):
    """
    A class representing the page start info.

    Refer: https://developers.facebook.com/docs/graph-api/reference/page-start-info/
    """
    date = attrib(default=None, type=Optional[PageStartDate])
    type = attrib(default=None, type=Optional[str])


@attrs
class Page(BaseModel):
    """
    A class representing the page info.

    Refer: https://developers.facebook.com/docs/graph-api/reference/page/
    """

    id = attrib(default=None, type=Optional[str])
    about = attrib(default=None, type=Optional[str], repr=False)
    can_checkin = attrib(default=None, type=Optional[bool], repr=False)
    category = attrib(default=None, type=Optional[str], repr=False)
    category_list = attrib(default=None, type=Optional[List[PageCategory]], repr=False)
    checkins = attrib(default=None, type=Optional[int], repr=False)
    contact_address = attrib(default=None, type=Optional[ContactAddress], repr=False)
    cover = attrib(default=None, type=Optional[CoverPhoto], repr=False)
    current_location = attrib(default=None, type=Optional[str], repr=False)
    description = attrib(default=None, type=Optional[str], repr=False)
    description_html = attrib(default=None, type=Optional[str], repr=False)
    display_subtext = attrib(default=None, type=Optional[str], repr=False)
    emails = attrib(default=None, type=Optional[List[str]], repr=False)
    engagement = attrib(default=None, type=Optional[PageEngagement], repr=False)
    fan_count = attrib(default=None, type=Optional[int], repr=False)
    followers_count = attrib(default=None, type=Optional[int], repr=False)
    founded = attrib(default=None, type=Optional[str], repr=False)
    general_info = attrib(default=None, type=Optional[str], repr=False)
    global_brand_page_name = attrib(default=None, type=Optional[str], repr=False)
    global_brand_root_id = attrib(default=None, type=Optional[str], repr=False)
    link = attrib(default=None, type=Optional[str], repr=False)
    name = attrib(default=None, type=Optional[str])
    # name_with_location_descriptor = attrib(default=None, type=Optional[str], repr=False)
    phone = attrib(default=None, type=Optional[str], repr=False)
    picture = attrib(default=None, type=Optional[Dict], repr=False)
    rating_count = attrib(default=None, type=Optional[str], repr=False)
    single_line_address = attrib(default=None, type=Optional[str], repr=False)
    start_info = attrib(default=None, type=Optional[PageStartInfo], repr=False)
    talking_about_count = attrib(default=None, type=Optional[int], repr=False)
    username = attrib(default=None, type=Optional[str])
    verification_status = attrib(default=None, type=Optional[str], repr=False)
    website = attrib(default=None, type=Optional[str], repr=False)
    were_here_count = attrib(default=None, type=Optional[int], repr=False)
    whatsapp_number = attrib(default=None, type=Optional[str], repr=False)

    def __attrs_post_init__(self):
        if self.picture is not None and isinstance(self.picture, dict):
            picture = self.picture.get("data", {})
            self.picture = ProfilePictureSource.new_from_json_dict(picture)



"""
    Instagram Basic display Api impl
"""
from typing import Dict, Optional, Union, List, Tuple, Set

from pyfacebook.api.base import BaseApi
from pyfacebook.error import PyFacebookException, ErrorCode, ErrorMessage
from pyfacebook.models import AuthAccessToken, IgBasicUser, IgBasicMedia, IgBasicMediaChildren
from pyfacebook.utils import constant
from pyfacebook.utils.param_validation import enf_comma_separated


class IgBasicApi(BaseApi):
    GRAPH_URL = "https://graph.instagram.com/"
    DEFAULT_AUTHORIZATION_URL = "https://api.instagram.com/oauth/authorize"
    DEFAULT_EXCHANGE_ACCESS_TOKEN_URL = "https://api.instagram.com/oauth/access_token"

    DEFAULT_SCOPE = ["user_profile", "user_media"]

    def __init__(self,
                 app_id=None,
                 app_secret=None,
                 short_token=None,
                 long_term_token=None,
                 initial_access_token=True,
                 timeout=None,  # type: Optional[int]
                 sleep_on_rate_limit=False,  # type: bool
                 sleep_seconds_mapping=None,  # type: Optional[Dict]
                 proxies=None,  # type: Optional[dict]
                 debug_http=False  # type: bool
                 ):
        BaseApi.__init__(self,
                         app_id=app_id,
                         app_secret=app_secret,
                         short_token=short_token,
                         long_term_token=long_term_token,
                         timeout=timeout,
                         initial_access_token=initial_access_token,
                         sleep_on_rate_limit=sleep_on_rate_limit,
                         sleep_seconds_mapping=sleep_seconds_mapping,
                         proxies=proxies,
                         debug_http=debug_http
                         )

    @staticmethod
    def _generate_secret_proof(secret, access_token):  # type: (str, str) -> Optional[str]
        return None

    def get_app_token(self, return_json=False):
        raise PyFacebookException(ErrorMessage(
            code=ErrorCode.NOT_SUPPORT_METHOD,
            message="Method not support by this api."
        ))

    def get_token_info(self, input_token=None, return_json=False):
        raise PyFacebookException(ErrorMessage(
            code=ErrorCode.NOT_SUPPORT_METHOD,
            message="Method not support by this api."
        ))

    def exchange_insights_token(self, page_id, access_token=None):
        raise PyFacebookException(ErrorMessage(
            code=ErrorCode.NOT_SUPPORT_METHOD,
            message="Method not support by this api."
        ))

    def get_long_token(self, short_token, app_id=None, app_secret=None, return_json=False):
        # type: (str, Optional[str], Optional[str], bool) -> Optional[Union[AuthAccessToken, Dict]]
        """
        Exchange a short-lived Instagram User Access Token for a long-lived Instagram User Access Token.

        :param short_token: short-lived Instagram User Access Token.
        :param app_id: Your Instagram app's id, this not necessary.
        :param app_secret: Your Instagram app's secret.
        :param return_json: Set to false will return instance of AuthAccessToken.
        Or return json data. Default is false.
        :return: Access token data.
        """
        if app_secret is None:
            app_secret = self.app_secret

        resp = self._request(
            path="access_token",
            args={
                "grant_type": "ig_exchange_token",
                "client_secret": app_secret,
                "access_token": short_token,
            },
            enforce_auth=False
        )
        data = self._parse_response(resp)

        if return_json:
            return data
        else:
            return AuthAccessToken.new_from_json_dict(data)

    def refresh_access_token(self, access_token=None, return_json=False):
        # type: (Optional[str], bool) -> Optional[Union[AuthAccessToken, Dict]]
        """
        :param access_token: long-lived token will be refreshed.
        :param return_json: Set to false will return instance of AuthAccessToken.
        Or return json data. Default is false.
        :return: New long-lived access token data
        """
        if access_token is None:
            access_token = self._access_token

        resp = self._request(
            path="refresh_access_token",
            args={
                "grant_type": "ig_refresh_token",
                "access_token": access_token
            }
        )
        data = self._parse_response(resp)

        if return_json:
            return data
        else:
            return AuthAccessToken.new_from_json_dict(data)

    def get_user_info(self, user_id=None, fields=None, return_json=False):
        # type: (str, Optional[Union[str, List, Tuple, Set]], bool) -> Union[IgBasicUser, Dict]
        """
        Retrieve user basic info for target user.
        :param user_id: The id for you want to get data. If not provide, will use access token belong user
        :param fields: Comma-separated id string for data fields which you want.
                You can also pass this with an id list, tuple, set.
        :param return_json: Set to false will return a list instance of IgBasicUser.
                Or return json data. Default is false.
        :return: User info data
        """

        if user_id is None:
            user_id = "me"

        if fields is None:
            fields = constant.INSTAGRAM_BASIC_USER_FIELD

        args = {
            "fields": enf_comma_separated(field="fields", value=fields)
        }

        resp = self._request(
            path="{}".format(user_id),
            args=args
        )
        data = self._parse_response(resp)

        if return_json:
            return data
        else:
            return IgBasicUser.new_from_json_dict(data)

    def paged_by_cursor(self,
                        target,  # type: str
                        resource,  # type: str
                        args,  # type: Dict
                        next_cursor=None  # type: Optional[str]
                        ):
        # type: (...) -> (Optional[str], Optional[str], Dict)
        """
        Paging data by cursor.

        :param target: Id for target object
        :param resource: resource type string.
        :param args: args dict.
        :param next_cursor: The paging cursor str. It will return from the api.
        :return: next cursor, previous cursor, data list
        """

        if next_cursor is not None:
            args["after"] = next_cursor

        resp = self._request(
            path="{0}/{1}".format(target, resource),
            args=args
        )

        next_cursor, previous_cursor = None, None
        data = self._parse_response(resp)

        if "paging" in data:
            paging = data["paging"]
            if "next" in paging:
                cursors = paging.get("cursors", {})
                next_cursor = cursors.get('after')
                previous_cursor = cursors.get('before')
        return next_cursor, previous_cursor, data

    def get_user_medias(self,
                        user_id=None,  # type: str
                        fields=None,  # type: Optional[Union[str, List, Tuple, Set]]
                        count=25,  # type: Optional[int]
                        limit=25,  # type: int
                        since=None,  # type: Optional[str]
                        until=None,  # type: Optional[str]
                        return_json=False,  # type: bool
                        ):
        # type: (...) -> List[Union[IgBasicMedia, Dict]]
        """
        Retrieve target user's medias.

        :param user_id: The id for you want to get data. If not provide, will use access token belong user.
        :param fields: Comma-separated id string for data fields which you want.
                You can also pass this with an id list, tuple, set.
        :param count: The count is you want to retrieve medias. Default is 25.
                If you want to get all data. Set it to None.
        :param limit: Each request retrieve posts count from api.
                For medias it may no more than 100. (Not verified.)
        :param since: A Unix timestamp that points to the start of a range of time-based data.
        :param until: A Unix timestamp that points to the end of a range of time-based data.
        :param return_json: Set to false will return a list instance of IgBasicMedia.
                Or return json data. Default is false.
        :return: Media data list
        """

        if user_id is None:
            user_id = "me"

        if fields is None:
            fields = constant.INSTAGRAM_BASIC_MEDIA_FIELD

        if count is not None:
            limit = min(count, limit)

        args = {
            "fields": enf_comma_separated(field="fields", value=fields),
            "since": since,
            "until": until,
            "limit": limit,
        }

        medias = []
        next_cursor = None

        while True:
            next_cursor, previous_cursor, data = self.paged_by_cursor(
                target=user_id,
                resource="media",
                args=args,
                next_cursor=next_cursor
            )
            data = data.get("data", [])

            if return_json:
                medias += data
            else:
                medias += [IgBasicMedia.new_from_json_dict(item) for item in data]

            if count is not None:
                if len(medias) >= count:
                    medias = medias[:count]
                    break
            if next_cursor is None:
                break

        return medias

    def get_media_info(self,
                       media_id,  # type: str
                       fields=None,  # type: Optional[Union[str, List, Tuple, Set]]
                       return_json=False,  # type: bool
                       ):
        # type: (...) -> Union[IgBasicMedia, Dict]
        """
        Retrieve media info for target media id.

        :param media_id: The id for target media id.
        :param fields: Comma-separated id string for data fields which you want.
                You can also pass this with an id list, tuple, set.
        :param return_json: Set to false will return an instance of IgBasicMedia.
                Or return json data. Default is false.
        :return: media data info.
        """

        if fields is None:
            fields = constant.INSTAGRAM_BASIC_MEDIA_FIELD

        args = {
            "fields": enf_comma_separated(field="fields", value=fields),
        }

        resp = self._request(
            path="{}".format(media_id),
            args=args
        )
        data = self._parse_response(resp)

        if return_json:
            return data
        else:
            return IgBasicMedia.new_from_json_dict(data)

    def get_media_children(self,
                           media_id,  # type: str
                           fields=None,  # type: Optional[Union[str, List, Tuple, Set]]
                           return_json=False,  # type: bool
                           ):
        # type: (...) -> List[Union[IgBasicMediaChildren, Dict]]
        """
        Retrieve media children data for target media.
        :param media_id: The id for target media.
        :param fields: Comma-separated id string for data fields which you want.
                You can also pass this with an id list, tuple, set.
        :param return_json: Set to false will return a list of instance of IgBasicMediaChildren.
                Or return json data. Default is false.
        :return: media children list
        """

        if fields is None:
            fields = constant.INSTAGRAM_BASIC_MEDIA_CHILDREN_FIELD

        args = {
            "fields": enf_comma_separated(field="fields", value=fields),
        }

        resp = self._request(
            path="{}/children".format(media_id),
            args=args
        )
        data = self._parse_response(resp)

        if return_json:
            return data["data"]
        else:
            return [IgBasicMediaChildren.new_from_json_dict(item) for item in data["data"]]



import json
import unittest

import pyfacebook.models as models


class IgBasicModelTest(unittest.TestCase):
    BASE_PATH = "testdata/instagram_basic/models/"

    with open(BASE_PATH + 'user_info.json', 'rb') as f:
        USER_INFO = json.loads(f.read().decode('utf-8'))
    with open(BASE_PATH + 'media_info.json', 'rb') as f:
        MEDIA_INFO = json.loads(f.read().decode("utf-8"))

    def testBasicUser(self):
        m = models.IgBasicUser.new_from_json_dict(self.USER_INFO)

        self.assertEqual(m.account_type, "BUSINESS")
        self.assertEqual(m.media_count, 10)

    def testBasicMedia(self):
        m = models.IgBasicMedia.new_from_json_dict(self.MEDIA_INFO)

        self.assertEqual(m.id, "18027939643230671")
        self.assertEqual(m.media_type, "CAROUSEL_ALBUM")
        self.assertEqual(len(m.children), 2)
        self.assertEqual(m.children[0].id, "17895013408404508")
        self.assertEqual(m.children[0].media_type, "IMAGE")



import json
import unittest

import pyfacebook.models as models


class PageModelTest(unittest.TestCase):
    BASE_PATH = "testdata/facebook/models/pages/"

    with open(BASE_PATH + 'page_categories.json', 'rb') as f:
        PAGE_CATEGORIES = json.loads(f.read().decode('utf-8'))
    with open(BASE_PATH + 'pages.json', 'rb') as f:
        PAGE_INFO = json.loads(f.read().decode("utf-8"))

    def testPageCategories(self):
        m = models.PageCategory.new_from_json_dict(self.PAGE_CATEGORIES)

        self.assertEqual(m.id, "1500")
        self.assertEqual(len(m.fb_page_categories), 3)
        self.assertEqual(m.fb_page_categories[2].fb_page_categories[1].api_enum, "CHARITY_ORGANIZATION")
        self.assertEqual(m.fb_page_categories[2].api_enum, "VISUAL_ARTS")

    def testPage(self):
        m = models.Page.new_from_json_dict(self.PAGE_INFO)

        self.assertTrue(isinstance(m, models.Page))
        self.assertEqual(m.id, "20531316728")
        self.assertEqual(m.can_checkin, True)
        self.assertEqual(len(m.category_list), 2)
        self.assertEqual(m.category_list[1].name, "Company")
        self.assertEqual(m.cover.id, "10159027219496729")
        self.assertEqual(m.engagement.count, 214840680)
        self.assertEqual(m.fan_count, 214840680)
        self.assertEqual(m.start_info.type, "Unspecified")
        self.assertEqual(m.start_info.date.year, 2004)
        self.assertEqual(m.were_here_count, 144228)



import json
import unittest

import responses
from six import iteritems

import pyfacebook


class VideoApiTest(unittest.TestCase):
    BASE_PATH = "testdata/facebook/apidata/videos/"
    BASE_URL = "https://graph.facebook.com/v8.0/"

    with open(BASE_PATH + "page_videos_p1.json", "rb") as f:
        VIDEOS_PAGED_1 = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "page_videos_p2.json", "rb") as f:
        VIDEOS_PAGED_2 = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "video_info.json", "rb") as f:
        VIDEO_INFO = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "videos_info.json", "rb") as f:
        VIDEOS_INFO = json.loads(f.read().decode("utf-8"))

    with open(BASE_PATH + "video_caption.json", "rb") as f:
        VIDEO_CAPTION_INFO = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "video_captions.json", "rb") as f:
        VIDEO_CAPTIONS_INFO = json.loads(f.read().decode("utf-8"))

    def setUp(self):
        self.api = pyfacebook.Api(
            app_id="12345678",
            app_secret="secret",
            long_term_token="token",
            version="v8.0"
        )

    def testGetVideosByObject(self):
        page_id = "367152833370567"
        # Test count
        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL + page_id + "/videos", json=self.VIDEOS_PAGED_1)

            videos = self.api.get_videos_by_object(
                object_id=page_id,
                count=3,
                limit=5,
                return_json=True,
            )
            self.assertEqual(len(videos), 3)

        # Test count is None
        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL + page_id + "/videos", json=self.VIDEOS_PAGED_1)
            m.add("GET", self.BASE_URL + page_id + "/videos", json=self.VIDEOS_PAGED_2)

            videos = self.api.get_videos_by_object(
                object_id=page_id,
                count=None,
                limit=5,
            )
            self.assertEqual(len(videos), 9)

    def testGetVideoInfo(self):
        video_id = "320504219400220"
        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL + video_id, json=self.VIDEO_INFO)

            video = self.api.get_video_info(
                video_id=video_id,
                fields=['id',
                        'description',
                        'created_time',
                        'embed_html',
                        'embeddable',
                        'is_crosspost_video',
                        'is_crossposting_eligible',
                        'is_episode',
                        'is_instagram_eligible',
                        'length',
                        'live_status',
                        'permalink_url',
                        'picture',
                        'published',
                        'status',
                        'updated_time',
                        "likes.summary(true).limit(0)",
                        "comments.summary(true).limit(0)"]
            )
            self.assertEqual(video.id, video_id)
            self.assertEqual(video.likes.total_count, 1525)

            video_json = self.api.get_video_info(
                video_id=video_id,
                return_json=True
            )
            self.assertEqual(video_json["id"], video_id)

    def testGetVideos(self):
        ids = ["320504219400220", "1237122236642185"]
        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL, json=self.VIDEOS_INFO)

            video_dict = self.api.get_videos(
                ids=ids,
            )
            for _id, data in iteritems(video_dict):
                self.assertIn(_id, ids)
                self.assertEqual(_id, data.id)

        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL, json=self.VIDEOS_INFO)
            video_dict = self.api.get_videos(
                ids=ids,
                fields=["id", "description", "content_category", "created_time"],
                return_json=True
            )
            for _id, data in iteritems(video_dict):
                self.assertIn(_id, ids)
                self.assertEqual(_id, data["id"])

    def testGetCaptionByVideoID(self):
        video_id = "969222676905304"
        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL + video_id + "/captions", json=self.VIDEO_CAPTION_INFO)

            captions = self.api.get_caption_by_video_id(
                video_id=video_id
            )
            self.assertEqual(len(captions), 1)
            self.assertEqual(captions[0].create_time, "1599928066")

            captions_json = self.api.get_caption_by_video_id(
                video_id=video_id,
                return_json=True
            )
            self.assertEqual(len(captions_json["data"]), 1)

    def testGetCaptionByVideoIDs(self):
        ids = ["969222676905304", "210174653594254"]

        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL + "captions", json=self.VIDEO_CAPTIONS_INFO)

            captions_dict = self.api.get_caption_by_video_ids(
                ids=ids,
            )
            for _id, data in iteritems(captions_dict):
                self.assertIn(_id, ids)

        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL + "captions", json=self.VIDEO_CAPTIONS_INFO)
            video_dict = self.api.get_caption_by_video_ids(
                ids=ids,
                return_json=True
            )
            for _id, data in iteritems(video_dict):
                self.assertIn(_id, ids)



"""
    These are some models for access token.
"""
from typing import List, Optional
from attr import attrs, attrib

from .base import BaseModel
from .._compat import str


@attrs
class AuthAccessToken(BaseModel):
    """
    A class representing the auth access token response.

    Refer: https://developers.facebook.com/docs/facebook-login/manually-build-a-login-flow#confirm
    """

    access_token = attrib(default=None, type=Optional[str])
    token_type = attrib(default=None, type=Optional[str])
    expires_in = attrib(default=None, type=Optional[int])
    expires_at = attrib(default=None, type=Optional[int], repr=False)


@attrs
class TokenError(BaseModel):
    """
    A class representing the access token error.

    https://developers.facebook.com/docs/graph-api/reference/v5.0/debug_token#Fields
    """
    code = attrib(default=None, type=Optional[int])
    message = attrib(default=None, type=Optional[str])
    sub_code = attrib(default=None, type=Optional[int], repr=False)


@attrs
class Metadata(BaseModel):
    """
    A class representing the access token metadata.

    https://developers.facebook.com/docs/graph-api/reference/v5.0/debug_token#Fields
    """
    sso = attrib(default=None, type=Optional[str])
    auth_type = attrib(default=None, type=Optional[str], repr=False)
    auth_nonce = attrib(default=None, type=Optional[str], repr=False)


@attrs
class GranularScope(BaseModel):
    """
    A class representing the access token granular scope.

    https://developers.facebook.com/docs/graph-api/reference/v5.0/debug_token#Fields
    """
    scope = attrib(default=None, type=Optional[str])
    target_ids = attrib(default=None, type=Optional[List[str]], repr=False)


@attrs
class AccessToken(BaseModel):
    """
    A class representing the access token.

    Refer: https://developers.facebook.com/docs/graph-api/reference/v4.0/debug_token
    """

    app_id = attrib(default=None, type=Optional[str])
    application = attrib(default=None, type=Optional[str])
    type = attrib(default=None, type=Optional[str], repr=False)
    error = attrib(default=None, type=Optional[TokenError], repr=False)
    expires_at = attrib(default=None, type=Optional[int], repr=False)
    data_access_expires_at = attrib(default=None, type=Optional[int], repr=False)
    is_valid = attrib(default=None, type=Optional[bool], repr=False)
    issued_at = attrib(default=None, type=Optional[int], repr=False)
    metadata = attrib(default=None, type=Optional[Metadata], repr=False)
    profile_id = attrib(default=None, type=Optional[str], repr=False)
    scopes = attrib(default=None, type=Optional[List[str]], repr=False)
    granular_scopes = attrib(default=None, type=Optional[List[GranularScope]], repr=False)
    user_id = attrib(default=None, type=Optional[str])



"""
    some models for common.
"""
from attr import attrib, attrs
from typing import Dict, List, Optional

from .base import BaseModel
from .picture import ImageSource
from .._compat import str


@attrs
class EntityAtTextRange(BaseModel):
    """
    A class representing the entity at text range info.

    Refer: https://developers.facebook.com/docs/graph-api/reference/entity-at-text-range/
    """
    id = attrib(default=None, type=Optional[str])
    length = attrib(default=None, type=Optional[int], repr=False)
    name = attrib(default=None, type=Optional[str])
    offset = attrib(default=None, type=Optional[int], repr=False)
    type = attrib(default=None, type=Optional[str])


@attrs
class StoryAttachmentMedia(BaseModel):
    """
    A class representing the post attachment media info.

    Refer: https://developers.facebook.com/docs/graph-api/reference/story-attachment-media/
    """
    image = attrib(default=None, type=Optional[ImageSource])
    source = attrib(default=None, type=Optional[str], repr=False)


@attrs
class StoryAttachmentTarget(BaseModel):
    """
    A class representing the post attachment target info.

    Refer: https://developers.facebook.com/docs/graph-api/reference/story-attachment-target/
    """
    id = attrib(default=None, type=Optional[str])
    unshimmed_url = attrib(default=None, type=Optional[str], repr=False)
    url = attrib(default=None, type=Optional[str], repr=False)


@attrs
class StoryAttachment(BaseModel):
    """
    A class representing the post attachment info.

    Refer: https://developers.facebook.com/docs/graph-api/reference/story-attachment/
    """
    description = attrib(default=None, type=Optional[str])
    description_tags = attrib(default=None, type=Optional[List[EntityAtTextRange]], repr=False)
    media = attrib(default=None, type=Optional[StoryAttachmentMedia], repr=False)
    media_type = attrib(default=None, type=Optional[str])
    target = attrib(default=None, type=Optional[StoryAttachmentTarget], repr=False)
    title = attrib(default=None, type=Optional[str], repr=False)
    type = attrib(default=None, type=Optional[str])
    unshimmed_url = attrib(default=None, type=Optional[str], repr=False)
    url = attrib(default=None, type=Optional[str], repr=False)
    subattachments = attrib(default=None, type=Optional[Dict], repr=False)

    def __attrs_post_init__(self):
        if self.subattachments is not None and isinstance(self.subattachments, dict):
            subattachments = self.subattachments.get("data", [])
            self.subattachments = [StoryAttachment.new_from_json_dict(item) for item in subattachments]


@attrs
class Privacy(BaseModel):
    """
    A class representing the privacy info.

    Refer: https://developers.facebook.com/docs/graph-api/reference/privacy/
    """
    allow = attrib(default=None, type=Optional[str], repr=False)
    deny = attrib(default=None, type=Optional[str], repr=False)
    description = attrib(default=None, type=Optional[str])
    friends = attrib(default=None, type=Optional[str], repr=False)
    networks = attrib(default=None, type=Optional[str], repr=False)
    value = attrib(default=None, type=Optional[str])


@attrs
class Location(BaseModel):
    """
    A class representing the location info.

    Refer: https://developers.facebook.com/docs/graph-api/reference/location/
    """
    city = attrib(default=None, type=Optional[str])
    country = attrib(default=None, type=Optional[str], repr=False)
    latitude = attrib(default=None, type=Optional[float], repr=False)
    located_in = attrib(default=None, type=Optional[str], repr=False)
    longitude = attrib(default=None, type=Optional[float], repr=False)
    state = attrib(default=None, type=Optional[str], repr=False)
    street = attrib(default=None, type=Optional[str], repr=False)
    zip = attrib(default=None, type=Optional[str], repr=False)


@attrs
class Place(BaseModel):
    id = attrib(default=None, type=Optional[str])
    name = attrib(default=None, type=Optional[str])
    location = attrib(default=None, type=Optional[Location], repr=False)
    overall_rating = attrib(default=None, type=Optional[float], repr=False)



"""
    These are models for comment.
"""

from attr import attrs, attrib
from typing import Optional

from .base import BaseModel
from .common import StoryAttachment
from .._compat import str


@attrs
class CommentSummary(BaseModel):
    """
    A class representing the comment summary info.

    Refer: https://developers.facebook.com/docs/graph-api/reference/post/comments/
    """
    order = attrib(default=None, type=Optional[str], repr=False)
    total_count = attrib(default=None, type=Optional[int])
    can_comment = attrib(default=None, type=Optional[bool])


@attrs
class Comment(BaseModel):
    """
    A class representing the comment info.

    Refer: https://developers.facebook.com/docs/graph-api/reference/v4.0/comment
    """

    # base fields
    id = attrib(default=None, type=Optional[str])
    can_like = attrib(default=None, type=Optional[bool], repr=False)
    can_comment = attrib(default=None, type=Optional[bool], repr=False)
    comment_count = attrib(default=None, type=Optional[int])
    created_time = attrib(default=None, type=Optional[str], repr=False)
    like_count = attrib(default=None, type=Optional[int])
    message = attrib(default=None, type=Optional[str], repr=False)
    permalink_url = attrib(default=None, type=Optional[str], repr=False)

    # connections fields
    attachment = attrib(default=None, type=Optional[StoryAttachment], repr=False)



import unittest

import pyfacebook
from pyfacebook.utils.param_validation import enf_comma_separated


class ParamValidationTest(unittest.TestCase):
    def testEnfCommaSeparated(self):
        self.assertEqual(enf_comma_separated("fields", "f1"), "f1")
        self.assertEqual(enf_comma_separated("fields", ["f1", "f2"]), "f1,f2")
        self.assertEqual(enf_comma_separated("fields", ["f1", "f2", "f2"]), "f1,f2")
        self.assertEqual(enf_comma_separated("fields", ("f1", "f2")), "f1,f2")
        self.assertTrue(enf_comma_separated("fields", {"f1", "f2"}))

        with self.assertRaises(pyfacebook.PyFacebookError):
            enf_comma_separated("id", 1)
        with self.assertRaises(pyfacebook.PyFacebookError):
            enf_comma_separated("id", [None, None])



"""
   These some models for facebook picture entity.
"""
from attr import attrs, attrib
from typing import Optional

from .base import BaseModel
from .._compat import str


@attrs
class CoverPhoto(BaseModel):
    """
    A class representing the cover photo info.

    Refer: https://developers.facebook.com/docs/graph-api/reference/cover-photo/
    """
    id = attrib(default=None, type=Optional[str])
    cover_id = attrib(default=None, type=Optional[str], repr=False)
    offset_x = attrib(default=None, type=Optional[float], repr=False)
    offset_y = attrib(default=None, type=Optional[float], repr=False)
    source = attrib(default=None, type=Optional[str])


@attrs
class ProfilePictureSource(BaseModel):
    """
    A class representing the profile picture source info

    Refer: https://developers.facebook.com/docs/graph-api/reference/profile-picture-source/
    """

    cache_key = attrib(default=None, type=Optional[str], repr=False)
    url = attrib(default=None, type=Optional[str])
    height = attrib(default=None, type=Optional[int])
    width = attrib(default=None, type=Optional[int])
    is_silhouette = attrib(default=None, type=Optional[bool], repr=False)


@attrs
class ImageSource(BaseModel):
    """
    A class representing the image source info.

    Structure will be {"height": 10, "width": 10, "src": "https://xxxx"}
    """
    height = attrib(default=None, type=Optional[int], repr=False)
    width = attrib(default=None, type=Optional[int], repr=False)
    src = attrib(default=None, type=Optional[str])



"""
    These are models for album entity.

    Refer: https://developers.facebook.com/docs/graph-api/reference/album
"""

from attr import attrs, attrib
from typing import Dict, Optional

from .base import BaseModel
from .common import Place
from .mixins import LikesAndCommentsSummaryField
from .._compat import str


@attrs
class Album(BaseModel, LikesAndCommentsSummaryField):
    """
    A class representing the album info.
    """

    id = attrib(default=None, type=Optional[str])
    backdated_time = attrib(default=None, type=Optional[str], repr=False)
    backdated_time_granularity = attrib(default=None, type=Optional[str], repr=False)
    can_upload = attrib(default=None, type=Optional[bool], repr=False)
    count = attrib(default=None, type=Optional[int], repr=False)
    cover_photo = attrib(default=None, type=Optional[Dict], repr=False)
    created_time = attrib(default=None, type=Optional[str])
    description = attrib(default=None, type=Optional[str], repr=False)
    # event = None
    # TODO now cattr not allow rename
    # from
    link = attrib(default=None, type=Optional[str], repr=False)
    location = attrib(default=None, type=Optional[str], repr=False)
    name = attrib(default=None, type=Optional[str])
    place = attrib(default=None, type=Optional[Place], repr=False)
    privacy = attrib(default=None, type=Optional[str], repr=False)
    type = attrib(default=None, type=Optional[str], repr=False)
    updated_time = attrib(default=None, type=Optional[str], repr=False)

    def __attrs_post_init__(self):
        super(Album, self).__attrs_post_init__()
        if self.cover_photo is not None:
            from .photo import Photo
            self.cover_photo = Photo.new_from_json_dict(self.cover_photo)



"""
    This is the base model
"""
import json

import cattr
from attr import attrs


@attrs
class BaseModel(object):

    @classmethod
    def new_from_json_dict(cls, data):
        instance = cattr.structure(data, cls)
        instance._json = data  # save original data
        return instance

    def as_dict(self):
        return cattr.unstructure(self)

    def as_json_str(self):
        return json.dumps(self.as_dict())



import json
import unittest

import responses

import pyfacebook


class IgBasicApiTest(unittest.TestCase):
    BASE_PATH = "testdata/instagram_basic/apidata/"

    with open(BASE_PATH + "long_access_token_info.json", "rb") as f:
        LONG_ACCESS_TOKEN_INFO = json.loads(f.read().decode("utf-8"))

    def setUp(self):
        self.api = pyfacebook.IgBasicApi(
            app_id="app id",
            app_secret="app secret",
            initial_access_token=False,
        )

    def testNotSupportMethod(self):
        self.assertIsNone(self.api._generate_secret_proof("secret", "token"))
        with self.assertRaises(pyfacebook.PyFacebookException):
            self.api.get_app_token()

        with self.assertRaises(pyfacebook.PyFacebookException):
            self.api.get_token_info(input_token="token")

        with self.assertRaises(pyfacebook.PyFacebookException):
            self.api.exchange_insights_token(page_id="page id", access_token="token")

    def testGetLongToken(self):
        with responses.RequestsMock() as m:
            m.add("GET", self.api.GRAPH_URL + "access_token", json=self.LONG_ACCESS_TOKEN_INFO)

            token = self.api.get_long_token(
                short_token="token"
            )
            self.assertEqual(token.access_token, "access-token")

            token_json = self.api.get_long_token(
                short_token="other token",
                app_secret="secret",
                return_json=True
            )
            self.assertEqual(token_json["expires_in"], 7200)

    def testGetRefreshToken(self):
        api = pyfacebook.IgBasicApi(long_term_token="token")

        with responses.RequestsMock() as m:
            m.add("GET", self.api.GRAPH_URL + "refresh_access_token", json=self.LONG_ACCESS_TOKEN_INFO)

            token = api.refresh_access_token()
            self.assertEqual(token.access_token, "access-token")

            token_json = api.refresh_access_token(
                access_token=self.api._access_token,
                return_json=True
            )
            self.assertEqual(token_json["access_token"], "access-token")



import sys

# -------
# Pythons
# -------

# Syntax sugar.
_ver = sys.version_info

#: Python 2.x?
is_py2 = (_ver.major == 2)

#: Python 3.x?
is_py3 = (_ver.major == 3)

if is_py2:
    str = unicode  # pragma: no cover # noqa

elif is_py3:
    str = str  # pragma: no cover



import json
import unittest

import responses
from six import iteritems

import pyfacebook


class PhotoApiTest(unittest.TestCase):
    BASE_PATH = "testdata/facebook/apidata/photos/"
    BASE_URL = "https://graph.facebook.com/v8.0/"

    with open(BASE_PATH + "photos_page_1.json", "rb") as f:
        PHOTO_PAGED_1 = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "photos_page_2.json", "rb") as f:
        PHOTO_PAGED_2 = json.loads(f.read().decode("utf-8"))

    with open(BASE_PATH + "photo_info.json", "rb") as f:
        PHOTO_INFO = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "photos_info.json", "rb") as f:
        PHOTOS_INFO = json.loads(f.read().decode("utf-8"))

    def setUp(self):
        self.api = pyfacebook.Api(
            app_id="12345678",
            app_secret="secret",
            long_term_token="token",
            version="v8.0"
        )

    def testGetPhotosByObject(self):
        object_id = "145161477045453"

        # Test count
        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL + object_id + "/photos", json=self.PHOTO_PAGED_1)

            photos = self.api.get_photos_by_object(
                object_id=object_id,
                count=3,
                limit=5,
                return_json=True,
            )
            self.assertEqual(len(photos), 3)

        # Test count is None
        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL + object_id + "/photos", json=self.PHOTO_PAGED_1)
            m.add("GET", self.BASE_URL + object_id + "/photos", json=self.PHOTO_PAGED_2)

            photos = self.api.get_photos_by_object(
                object_id=object_id,
                count=None,
                limit=5,
            )
            self.assertEqual(len(photos), 10)

    def testGetPhotoInfo(self):
        photo_id = "166370841591183"

        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL + photo_id, json=self.PHOTO_INFO)

            photo = self.api.get_photo_info(
                photo_id=photo_id,
            )
            self.assertEqual(photo.id, photo_id)
            self.assertEqual(photo.album.id, "108824087345859")

            photo_json = self.api.get_photo_info(
                photo_id=photo_id,
                return_json=True
            )
            self.assertEqual(photo_json["id"], photo_id)

    def testGetPhotos(self):
        ids = ["166370841591183", "198814968346770"]

        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL, json=self.PHOTOS_INFO)

            photo_dict = self.api.get_photos(
                ids=ids,
            )
            for _id, data in iteritems(photo_dict):
                self.assertIn(_id, ids)
                self.assertEqual(_id, data.id)

        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL, json=self.PHOTOS_INFO)
            photo_dict = self.api.get_photos(
                ids=ids,
                return_json=True
            )
            for _id, data in iteritems(photo_dict):
                self.assertIn(_id, ids)
                self.assertEqual(_id, data["id"])



import json
import unittest

import responses

import pyfacebook


class ApiDiscoveryTest(unittest.TestCase):
    BASE_PATH = "testdata/instagram/apidata/discovery/"
    BASE_URL = "https://graph.facebook.com/{}/".format(pyfacebook.Api.VALID_API_VERSIONS[-1])

    with open(BASE_PATH + "user_default.json", "rb") as f:
        USER_DEFAULT = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "user_fields.json", "rb") as f:
        USER_FIELDS = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "media_default_p1.json", "rb") as f:
        MEDIA_DEFAULT_P1 = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "media_default_p2.json", "rb") as f:
        MEDIA_DEFAULT_P2 = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "media_fields.json", "rb") as f:
        MEDIA_FIELDS = json.loads(f.read().decode("utf-8"))

    def setUp(self):
        self.instagram_business_id = "123456789"
        self.api = pyfacebook.IgProApi(
            app_id="123456", app_secret="secret",
            long_term_token="token",
            instagram_business_id=self.instagram_business_id
        )

    def testDiscoveryUser(self):
        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL + self.instagram_business_id, json=self.USER_DEFAULT)
            m.add("GET", self.BASE_URL + self.instagram_business_id, json=self.USER_FIELDS)

            user_json = self.api.discovery_user(
                username="facebook",
                return_json=True,
            )

            self.assertEqual(user_json["id"], "17841400455970028")
            self.assertEqual(user_json["followers_count"], 3502538)

            user_model = self.api.discovery_user(
                username="facebook",
                fields=["id", "followers_count", "follows_count", "name", "username"]
            )
            self.assertEqual(user_model.id, "17841400455970028")
            self.assertEqual(user_model.followers_count, 3502538)

    def testDiscoveryMedia(self):
        # test for error time
        with self.assertRaises(pyfacebook.PyFacebookException):
            self.api.discovery_user_medias("facebook", since_time="2019")
            self.api.discovery_user_medias("facebook", until_time="2019")

        # test time interval all medias.
        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL + self.instagram_business_id, json=self.MEDIA_DEFAULT_P1)
            m.add("GET", self.BASE_URL + self.instagram_business_id, json=self.MEDIA_DEFAULT_P2)

            medias = self.api.discovery_user_medias(
                username="facebook",
                count=None,
                limit=5,
                since_time="2019-11-1",
                until_time="2019-12-20",
            )
            self.assertEqual(len(medias), 5)

        # test get all medias.
        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL + self.instagram_business_id, json=self.MEDIA_DEFAULT_P1)
            m.add("GET", self.BASE_URL + self.instagram_business_id, json=self.MEDIA_DEFAULT_P2)

            medias = self.api.discovery_user_medias(
                username="facebook",
                count=None,
                limit=5,
            )

            self.assertEqual(len(medias), 10)

        # test count and with fields
        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL + self.instagram_business_id, json=self.MEDIA_FIELDS)

            medias = self.api.discovery_user_medias(
                username="facebook",
                fields=["id", "caption", "comments_count", "like_count"],
                count=4,
                return_json=True
            )

            self.assertEqual(len(medias), 4)

        # test time and not timestamp field
        with self.assertRaises(pyfacebook.PyFacebookException):
            self.api.discovery_user_medias(
                username="facebook",
                fields=["id", "caption", "comments_count", "like_count"],
                since_time="2019-10-1",
            )



from .access_token import AccessToken, AuthAccessToken
from .base import BaseModel
from .comment import Comment, CommentSummary
from .page import Page, PageCategory
from .picture import ProfilePictureSource, CoverPhoto
from .post import Post
from .video import Video, VideoCaption
from .photo import Photo
from .album import Album
from .live import LiveVideo, LiveVideoInputStream
from .ig_basic_models import IgBasicUser, IgBasicMedia, IgBasicMediaChildren
from .ig_pro_models import (
    IgProUser, IgProMedia, IgProComment,
    IgProReply, IgProHashtag, IgProInsight, IgProStory,
    IgProContainer, IgProPublishLimit
)

__all__ = [
    "AccessToken",
    "AuthAccessToken",
    "BaseModel",
    "CoverPhoto",
    "Comment",
    "CommentSummary",
    "Page",
    "PageCategory",
    "Post",
    "ProfilePictureSource",
    "Video",
    "VideoCaption",
    "Photo",
    "Album",
    "LiveVideo",
    "LiveVideoInputStream",
    # Instagram Basic display
    "IgBasicUser",
    "IgBasicMedia",
    "IgBasicMediaChildren",
    # Instagram Professional
    "IgProUser",
    "IgProMedia",
    "IgProComment",
    "IgProReply",
    "IgProHashtag",
    "IgProInsight",
    "IgProStory",
    "IgProContainer",
    "IgProPublishLimit",
]



"""
    These are models for post entity.
"""

from attr import attrs, attrib
from typing import Optional, Dict, List

from .base import BaseModel
from .common import StoryAttachment
from .mixins import CommentsSummaryField
from .._compat import str


@attrs
class PostShares(BaseModel):
    """
    A class representing the page post shares info.

    structure is {"count": 12}
    """
    count = attrib(default=None, type=Optional[int])


@attrs
class ReactionsSummary(BaseModel):
    """
    A class representing the page post reaction summary info.

    Refer: https://developers.facebook.com/docs/graph-api/reference/page-post/reactions/
    """
    total_count = attrib(default=None, type=Optional[int])
    viewer_reaction = attrib(default=None, type=Optional[str])


@attrs
class MessageTag(BaseModel):
    """
    A class representing the page post message tag ingo.
    """
    id = attrib(default=None, type=Optional[str])
    name = attrib(default=None, type=Optional[str])
    offset = attrib(default=None, type=Optional[int])
    length = attrib(default=None, type=Optional[int])


@attrs
class Post(BaseModel, CommentsSummaryField):
    """
    A class representing the post info.

    Refer: https://developers.facebook.com/docs/graph-api/reference/post
    """

    # base fields
    id = attrib(default=None, type=Optional[str])
    backdated_time = attrib(default=None, type=Optional[str], repr=False)
    created_time = attrib(default=None, type=Optional[str], repr=False)
    full_picture = attrib(default=None, type=Optional[str], repr=False)
    icon = attrib(default=None, type=Optional[str], repr=False)
    message = attrib(default=None, type=Optional[str], repr=False)
    message_tags = attrib(default=None, type=Optional[List[MessageTag]], repr=False)
    permalink_url = attrib(default=None, type=Optional[str])
    shares = attrib(default=None, type=Optional[PostShares], repr=False)
    status_type = attrib(default=None, type=Optional[str], repr=False)
    story = attrib(default=None, type=Optional[str], repr=False)
    updated_time = attrib(default=None, type=Optional[str], repr=False)
    tagged_time = attrib(default=None, type=Optional[str], repr=False)  # This only for the tagged endpoint.
    # connections fields
    attachments = attrib(default=None, type=Optional[Dict], repr=False)
    comments = attrib(default=None, type=Optional[Dict], repr=False)
    reactions = attrib(default=None, type=Optional[Dict], repr=False)
    like = attrib(default=None, type=Optional[Dict], repr=False)
    love = attrib(default=None, type=Optional[Dict], repr=False)
    wow = attrib(default=None, type=Optional[Dict], repr=False)
    haha = attrib(default=None, type=Optional[Dict], repr=False)
    sad = attrib(default=None, type=Optional[Dict], repr=False)
    angry = attrib(default=None, type=Optional[Dict], repr=False)
    thankful = attrib(default=None, type=Optional[Dict], repr=False)

    def __attrs_post_init__(self):
        super(Post, self).__attrs_post_init__()

        if self.attachments is not None and isinstance(self.attachments, dict):
            attachments = self.attachments.get("data", [])
            self.attachments = [StoryAttachment.new_from_json_dict(item) for item in attachments]
        # handle the reaction items
        self.reactions_handler("reactions", init_type=False)
        self.reactions_handler("like")
        self.reactions_handler("love")
        self.reactions_handler("wow")
        self.reactions_handler("haha")
        self.reactions_handler("sad")
        self.reactions_handler("angry")
        self.reactions_handler("thankful")

    def reactions_handler(self, field_name, init_type=True):
        field_value = getattr(self, field_name)
        if field_value is not None and isinstance(field_value, dict):
            field_summary = field_value.get("summary", {})
            if init_type:
                if field_summary.get("viewer_reaction") is None:
                    field_summary["viewer_reaction"] = field_name
            field_value = ReactionsSummary.new_from_json_dict(field_summary)
            setattr(self, field_name, field_value)



"""
    tests for publish
"""

import json
import unittest

import responses

import pyfacebook


class ApiPublishTest(unittest.TestCase):
    BASE_PATH = "testdata/instagram/apidata/publish/"
    BASE_URL_WITHOUT_VERSION = "https://graph.facebook.com/"
    BASE_URL = BASE_URL_WITHOUT_VERSION + "{}/".format(pyfacebook.Api.VALID_API_VERSIONS[-1])

    with open(BASE_PATH + "container_info.json", "rb") as f:
        CONTAINER_INFO = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "create_photo.json", "rb") as f:
        CREATE_PHOTO = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "create_video.json", "rb") as f:
        CREATE_VIDEO = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "publish_container.json", "rb") as f:
        PUBLISH_CONTAINER = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "publish_limit_resp.json", "rb") as f:
        PUBLISH_LIMIT_RESP = json.loads(f.read().decode("utf-8"))

    def setUp(self):
        self.instagram_business_id = "17841406338772941"
        self.api = pyfacebook.IgProApi(
            app_id="123456", app_secret="secret",
            long_term_token="token",
            instagram_business_id=self.instagram_business_id,
        )

    def testCreatePhoto(self):
        with responses.RequestsMock() as m:
            m.add("POST", self.BASE_URL_WITHOUT_VERSION + self.instagram_business_id + "/media", json=self.CREATE_PHOTO)

            resp = self.api.create_photo(
                image_url="https//www.example.com/images/bronz-fonz.jpg",
                caption="BronzFonz",
            )
            self.assertEqual(resp.id, "17889455560051444")

            resp = self.api.create_photo(
                image_url="https//www.example.com/images/bronz-fonz.jpg",
                caption="BronzFonz",
                return_json=True,
            )
            self.assertEqual(resp["id"], "17889455560051444")

    def testCreateVideo(self):
        with responses.RequestsMock() as m:
            m.add("POST", self.BASE_URL_WITHOUT_VERSION + self.instagram_business_id + "/media", json=self.CREATE_VIDEO)

            resp = self.api.create_video(
                video_url="https://www.example.com/videos/hungry-fonzes.mov",
                caption="Heyyyyyyyy",
            )
            self.assertEqual(resp.id, "17889455560051447")

            resp = self.api.create_video(
                video_url="https://www.example.com/videos/hungry-fonzes.mov",
                caption="Heyyyyyyyy",
                return_json=True,
            )
            self.assertEqual(resp["id"], "17889455560051447")

    def testPublishContainer(self):
        with responses.RequestsMock() as m:
            m.add(
                "POST",
                self.BASE_URL_WITHOUT_VERSION + self.instagram_business_id + "/media_publish",
                json=self.PUBLISH_CONTAINER,
            )

            resp = self.api.publish_container(
                creation_id="17889455560051447",
            )
            self.assertEqual(resp["id"], "17920238422030506")

    def testGetContainerInfo(self):
        container_id = "17889455560051447"

        with responses.RequestsMock() as m:
            m.add(
                "GET",
                self.BASE_URL + container_id,
                json=self.CONTAINER_INFO,
            )

            resp = self.api.get_container_info(
                container_id=container_id,
            )
            self.assertEqual(resp.id, container_id)
            self.assertEqual(resp.status_code, "PUBLISHED")

            resp_json = self.api.get_container_info(
                container_id=container_id,
                fields="id,status_code",
                return_json=True
            )
            self.assertEqual(resp_json["id"], container_id)

    def testPublishLimit(self):
        with responses.RequestsMock() as m:
            m.add(
                "GET",
                self.BASE_URL + self.instagram_business_id + "/content_publishing_limit",
                json=self.PUBLISH_LIMIT_RESP,
            )

            resp = self.api.get_publish_limit()
            self.assertEqual(resp.quota_usage, 0)
            self.assertEqual(resp.config.quota_total, 25)

            resp_json = self.api.get_publish_limit(
                fields="quota_usage",
                return_json=True,
            )
            self.assertEqual(resp_json["data"][0]["quota_usage"], 0)



import json
import unittest

import pyfacebook.models as models


class AlbumModelTest(unittest.TestCase):
    BASE_PATH = "testdata/facebook/models/albums/"

    with open(BASE_PATH + 'album.json', 'rb') as f:
        ALBUM_INFO = json.loads(f.read().decode('utf-8'))

    def testAlbum(self):
        m = models.Album.new_from_json_dict(self.ALBUM_INFO)

        self.assertEqual(m.id, "145161477045453")
        self.assertEqual(m.cover_photo.id, "198814975013436")
        self.assertEqual(m.type, "wall")



import json
import unittest

import responses
import pyfacebook


class ApiMentionTest(unittest.TestCase):
    BASE_PATH = "testdata/instagram/apidata/mentions/"
    BASE_URL = "https://graph.facebook.com/{}/".format(pyfacebook.Api.VALID_API_VERSIONS[-1])

    with open(BASE_PATH + "tagged_user_medias_p1.json", "rb") as f:
        TAGGED_USER_MEDIAS_P1 = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "tagged_user_medias_p2.json", "rb") as f:
        TAGGED_USER_MEDIAS_P2 = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "mention_user_comment.json", "rb") as f:
        MENTION_USER_COMMENT = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "mention_user_media.json", "rb") as f:
        MENTION_USER_MEDIA = json.loads(f.read().decode("utf-8"))

    def setUp(self):
        self.instagram_business_id = "17841406338772941"
        self.api = pyfacebook.IgProApi(
            app_id="123456", app_secret="secret",
            long_term_token="token"
        )

    def testGetTaggedUserMedias(self):
        # test get all medias

        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL + self.instagram_business_id + "/tags", json=self.TAGGED_USER_MEDIAS_P1)
            m.add("GET", self.BASE_URL + self.instagram_business_id + "/tags", json=self.TAGGED_USER_MEDIAS_P2)

            res = self.api.get_tagged_user_medias(
                user_id=self.instagram_business_id,
                count=None,
                limit=3,
            )
            self.assertEqual(len(res), 5)

        # test count
        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL + self.instagram_business_id + "/tags", json=self.TAGGED_USER_MEDIAS_P1)
            m.add("GET", self.BASE_URL + self.instagram_business_id + "/tags", json=self.TAGGED_USER_MEDIAS_P2)

            res = self.api.get_tagged_user_medias(
                user_id=self.instagram_business_id,
                count=4,
                limit=3,
                access_token="token",
                return_json=True
            )
            self.assertEqual(len(res), 4)

    def testGetMentionedCommentInfo(self):
        comment_id = "17892250648466172"
        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL + self.instagram_business_id, json=self.MENTION_USER_COMMENT)

            comment = self.api.get_mentioned_comment_info(
                user_id=self.instagram_business_id,
                comment_id=comment_id,
            )
            self.assertEqual(comment.id, comment_id)
            self.assertEqual(comment.media.id, "17846368219941692")

            comment_json = self.api.get_mentioned_comment_info(
                user_id=self.instagram_business_id,
                comment_id=comment_id,
                access_token="token",
                return_json=True
            )
            self.assertEqual(comment_json["id"], comment_id)
            self.assertEqual(comment_json["media"]["id"], "17846368219941692")

    def testGetMentionedMediaInfo(self):
        media_id = "18027939643230671"

        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL + self.instagram_business_id, json=self.MENTION_USER_MEDIA)

            media = self.api.get_mentioned_media_info(
                user_id=self.instagram_business_id,
                media_id=media_id,
            )
            self.assertEqual(media.id, media_id)
            self.assertEqual(len(media.comments), 1)
            self.assertEqual(len(media.children), 2)

            media_json = self.api.get_mentioned_media_info(
                user_id=self.instagram_business_id,
                media_id=media_id,
                access_token="token",
                return_json=True,
            )
            self.assertEqual(media_json["id"], media_id)
            self.assertEqual(len(media_json["comments"]), 1)
            self.assertEqual(len(media_json["children"]["data"]), 2)



import json
import unittest

import responses
from six import iteritems

import pyfacebook


class AlbumApiTest(unittest.TestCase):
    BASE_PATH = "testdata/facebook/apidata/albums/"
    BASE_URL = "https://graph.facebook.com/v8.0/"

    with open(BASE_PATH + "albums_paged_1.json", "rb") as f:
        ALBUM_PAGED_1 = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "albums_paged_2.json", "rb") as f:
        ALBUM_PAGED_2 = json.loads(f.read().decode("utf-8"))

    with open(BASE_PATH + "album_info.json", "rb") as f:
        ALBUM_INFO = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "albums_info.json", "rb") as f:
        ALBUMS_INFO = json.loads(f.read().decode("utf-8"))

    def setUp(self):
        self.api = pyfacebook.Api(
            app_id="12345678",
            app_secret="secret",
            long_term_token="token",
            version="v8.0"
        )

    def testGetAlbumByObject(self):
        object_id = "367152833370567"

        # Test count
        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL + object_id + "/albums", json=self.ALBUM_PAGED_1)

            albums = self.api.get_albums_by_object(
                object_id=object_id,
                count=3,
                limit=5,
                return_json=True,
            )
            self.assertEqual(len(albums), 3)

        # Test count is None
        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL + object_id + "/albums", json=self.ALBUM_PAGED_1)
            m.add("GET", self.BASE_URL + object_id + "/albums", json=self.ALBUM_PAGED_2)

            albums = self.api.get_albums_by_object(
                object_id=object_id,
                count=None,
                limit=5,
            )
            self.assertEqual(len(albums), 10)

    def testGetAlbumInfo(self):
        album_id = "372558296163354"

        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL + album_id, json=self.ALBUM_INFO)

            album = self.api.get_album_info(
                album_id=album_id,
            )
            self.assertEqual(album.id, album_id)
            self.assertEqual(album.cover_photo.id, "3293405020745319")

            album_json = self.api.get_album_info(
                album_id=album_id,
                return_json=True
            )
            self.assertEqual(album_json["id"], album_id)

    def testGetAlbums(self):
        ids = ["372558296163354", "443259089093274"]

        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL, json=self.ALBUMS_INFO)

            album_dict = self.api.get_albums(
                ids=ids,
            )
            for _id, data in iteritems(album_dict):
                self.assertIn(_id, ids)
                self.assertEqual(_id, data.id)

        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL, json=self.ALBUMS_INFO)
            album_dict = self.api.get_albums(
                ids=ids,
                fields=["id", "description", "content_category", "created_time"],
                return_json=True
            )
            for _id, data in iteritems(album_dict):
                self.assertIn(_id, ids)
                self.assertEqual(_id, data["id"])



"""
    Facebook Graph Api impl
"""
from six import iteritems
from typing import Optional, Union, List, Tuple, Set

from pyfacebook.error import PyFacebookException, ErrorMessage, ErrorCode

from pyfacebook.models import (
    Page, Comment, CommentSummary,
    ProfilePictureSource, Post,
    Video, VideoCaption,
    Album, Photo, LiveVideo, LiveVideoInputStream
)
from pyfacebook.api.base import BaseApi
from pyfacebook.utils import constant
from pyfacebook.utils.param_validation import enf_comma_separated


class Api(BaseApi):
    def __init__(self,
                 app_id=None,
                 app_secret=None,
                 short_token=None,
                 long_term_token=None,
                 application_only_auth=False,
                 initial_access_token=True,  # type: bool
                 version=None,
                 timeout=None,
                 sleep_on_rate_limit=False,
                 sleep_seconds_mapping=None,
                 proxies=None,
                 debug_http=False
                 ):
        BaseApi.__init__(
            self,
            app_id=app_id,
            app_secret=app_secret,
            short_token=short_token,
            long_term_token=long_term_token,
            application_only_auth=application_only_auth,
            initial_access_token=initial_access_token,
            version=version,
            timeout=timeout,
            sleep_on_rate_limit=sleep_on_rate_limit,
            sleep_seconds_mapping=sleep_seconds_mapping,
            proxies=proxies,
            debug_http=debug_http
        )

    def get_page_info(self,
                      page_id=None,  # type: Optional[str]
                      username=None,  # type: Optional[str]
                      fields=None,  # type: Optional[Union[str, List, Tuple, Set]]
                      return_json=False  # type: bool
                      ):
        # type: (...) -> Union[Page, dict]
        """
        Retrieve the given page's basic info.

        :param page_id: The id for page.
        :param username: The username for page.
        :param fields:Comma-separated id string for data fields which you want.
        You can also pass this with an id list, tuple, set.
        :param return_json: Set to false will return instance of Page.
        Or return json data. Default is false.
        """
        if page_id:
            target = page_id
        elif username:
            target = username
        else:
            raise PyFacebookException(ErrorMessage(
                code=ErrorCode.MISSING_PARAMS,
                message="Specify at least one of page_id or username",
            ))
        if fields is None:
            fields = constant.FB_PAGE_FIELDS

        args = {
            "fields": enf_comma_separated("fields", fields)
        }

        resp = self._request(
            method='GET',
            path='{0}/{1}'.format(self.version, target),
            args=args
        )

        data = self._parse_response(resp)
        if return_json:
            return data
        else:
            return Page.new_from_json_dict(data)

    def get_pages_info(self,
                       ids,  # type: Optional[Union[str, List, Tuple, Set]]
                       fields=None,  # type: Optional[Union[str, List, Tuple, Set]]
                       return_json=False  # type: bool
                       ):
        # type: (...) -> dict
        """
        Retrieve multi pages info by one request.

        :param ids: Comma-separated id(username) string for page which you want to get.
        You can also pass this with an id list, tuple, set.
        :param fields:Comma-separated id string for data fields which you want.
        You can also pass this with an id list, tuple, set.
        :param return_json: Set to false will return a dict of Page instances.
        Or return json data. Default is false.
        """
        if fields is None:
            fields = constant.FB_PAGE_FIELDS

        args = {
            "ids": enf_comma_separated("ids", ids),
            "fields": enf_comma_separated("fields", fields)
        }
        resp = self._request(
            method='GET',
            path='{0}/'.format(self.version),
            args=args
        )

        data = self._parse_response(resp)
        if return_json:
            return data
        else:
            return {_id: Page.new_from_json_dict(p_data) for _id, p_data in iteritems(data)}

    def paged_by_cursor(self,
                        target,
                        resource,
                        args,
                        next_cursor=None):
        """
        Used cursor-based pagination.
        Refer: https://developers.facebook.com/docs/graph-api/using-graph-api/#paging

        Args:
             target (str)
                The id which target (page,user...) you want to retrieve data.
            resource (str)
                The resource string. just the connections (posts,comments and so on).
            args (dict)
                The params dict for the resource.
            next_cursor (str, optional)
                The paging cursor str. It will return from the graph api.
        Returns:
            The origin data return from the graph api.
        """
        if next_cursor is not None:
            resp = self._request(
                method='GET',
                path=next_cursor
            )
        else:
            resp = self._request(
                method='GET',
                path='{version}/{target}/{resource}'.format(
                    version=self.version, target=target, resource=resource
                ),
                args=args
            )
        _next, previous = None, None
        data = self._parse_response(resp)
        if 'paging' in data:
            _next = data['paging'].get('next')
            previous = data['paging'].get('previous')
        return _next, previous, data

    def get_page_feeds(self,
                       page_id,  # type: str
                       fields=None,  # type: Optional[Union[str, List, Tuple, Set]]
                       resource=None,  # type: Optional[str]
                       since_time=None,  # type: str
                       until_time=None,  # type: str
                       count=10,  # type: Optional[int]
                       limit=10,  # type: int
                       access_token=None,  # type: str
                       return_json=False  # type: bool
                       ):
        # type: (...) -> List[Union[Post, dict]]
        """
        Retrieve data for give page's posts info.

        Refer: https://developers.facebook.com/docs/graph-api/reference/v4.0/page/feed

        :param page_id: The id(username) for page you want to retrieve data.
        :param fields: Comma-separated id string for data fields which you want.
        You can also pass this with an id list, tuple, set.
        :param resource: The data endpoint type, may have posts,feed,tagged,published_posts.
        :param since_time: A Unix timestamp that points to the start of the range of time-based data.
        :param until_time: A Unix timestamp that points to the end of the range of time-based data.
        :param count: The count will retrieve posts. If you want to get all data. Set it to None.
        :param limit: Each request retrieve posts count from api. For posts it should no more than 100.
        :param access_token: If you want to pass with own access token, can with this parameter.
        :param return_json: Set to false will return a list of Page instances.
        Or return json data. Default is false.
        :return:
        """
        if resource is None:
            resource = 'feed'

        if fields is None:
            fields = constant.FB_POST_BASIC_FIELDS + constant.FB_POST_REACTIONS_FIELD

        args = {
            'fields': enf_comma_separated("fields", fields),
            'since': since_time,
            'until': until_time,
            'limit': limit,
        }

        if access_token is not None:
            args['access_token'] = access_token

        posts = []
        next_cursor = None

        while True:
            next_cursor, previous_cursor, data = self.paged_by_cursor(
                resource=resource,
                target=page_id,
                args=args,
                next_cursor=next_cursor,
            )
            if return_json:
                posts += data.get('data', [])
            else:
                posts += [Post.new_from_json_dict(item) for item in data['data']]
            if count is not None:
                if len(posts) >= count:
                    posts = posts[:count]
                    break
            if next_cursor is None:
                break
        return posts

    def get_page_posts(self,
                       page_id,  # type: str
                       fields=None,  # type: Optional[Union[str, List, Tuple, Set]]
                       since_time=None,  # type: str
                       until_time=None,  # type: str
                       count=10,  # type: Optional[int]
                       limit=10,  # type: int
                       return_json=False  # type: bool
                       ):
        # type: (...) -> List[Optional[Post, dict]]
        """
        Retrieve the give page's posts info.

        :param page_id: The id(username) for page you want to retrieve data.
        :param fields: Comma-separated id string for data fields which you want.
        You can also pass this with an id list, tuple, set.
        :param since_time: A Unix timestamp that points to the start of the range of time-based data.
        :param until_time: A Unix timestamp that points to the end of the range of time-based data.
        :param count: The count will retrieve posts. If you want to get all data. Set it to None.
        :param limit: Each request retrieve posts count from api. For posts it should no more than 100.
        :param return_json: Set to false will return a list of Post instances.
        Or return json data. Default is false.
        """
        return self.get_page_feeds(
            page_id, fields, 'posts',
            since_time, until_time,
            count, limit,
            return_json=return_json
        )

    def get_page_published_posts(self,
                                 page_id,  # type: str
                                 fields=None,  # type: Optional[Union[str, List, Tuple, Set]]
                                 since_time=None,  # type: str
                                 until_time=None,  # type: str
                                 count=10,  # type: Optional[int]
                                 limit=10,  # type: int
                                 access_token=None,  # type: str
                                 return_json=False  # type: bool
                                 ):
        # type: (...) -> List[Union[Post, dict]]
        """
        Retrieve the give page's all posts info.
        Note: This need a page access token with manage_pages permissions.

        :param page_id: The id for page you want to retrieve data.
        :param fields: Comma-separated id string for data fields which you want.
        You can also pass this with an id list, tuple, set.
        :param since_time: A Unix timestamp that points to the start of the range of time-based data.
        :param until_time: A Unix timestamp that points to the end of the range of time-based data.
        :param count: The count will retrieve posts. If you want to get all data. Set it to None.
        :param limit: Each request retrieve posts count from api. For posts it should no more than 100.
        :param access_token: If you want use other token to get data. you can provide with this.
        :param return_json: Set to false will return a list of Post instances.
        Or return json data. Default is false.
        """
        return self.get_page_feeds(
            page_id, fields, 'published_posts',
            since_time, until_time,
            count, limit,
            access_token=access_token,
            return_json=return_json
        )

    def get_page_tagged_posts(self,
                              page_id,  # type: str
                              fields=None,  # type: Optional[Union[str, List, Tuple, Set]]
                              since_time=None,  # type: str
                              until_time=None,  # type: str
                              count=10,  # type: Optional[int]
                              limit=10,  # type: int
                              access_token=None,  # type: str
                              return_json=False  # type: bool
                              ):
        # type: (...) -> List[Union[Post, dict]]
        """
        Obtain posts which tagged the given page. If token is authorized for app.
        This endpoint need the page token and has manage-pages.

        :param page_id: The id for page you want to retrieve data.
        :param fields: Comma-separated id string for data fields which you want.
        You can also pass this with an id list, tuple, set.
        :param since_time: A Unix timestamp that points to the start of the range of time-based data.
        :param until_time: A Unix timestamp that points to the end of the range of time-based data.
        :param count: The count will retrieve posts. If you want to get all data. Set it to None.
        :param limit: Each request retrieve posts count from api. For posts it should no more than 100.
        :param access_token: If you want use other token to get data. you can provide with this.
        :param return_json: Set to false will return a list of Post instances.
        Or return json data. Default is false.
        """
        return self.get_page_feeds(
            page_id, fields, 'tagged',
            since_time, until_time,
            count, limit,
            access_token=access_token,
            return_json=return_json
        )

    def get_post_info(self,
                      post_id,  # type: str
                      fields=None,  # type: Optional[Union[str, List, Tuple, Set]]
                      return_json=False  # type: bool
                      ):
        # type: (...) -> Optional[Post, dict]
        """
        Obtain give post's basic info.
        :param post_id: The id for post you want to retrieve data.
        :param fields: Comma-separated id string for data fields which you want.
        You can also pass this with an id list, tuple, set.
        :param return_json: Set to false will return a list of Post instances.
        Or return json data. Default is false.
        """
        if fields is None:
            fields = constant.FB_POST_BASIC_FIELDS + constant.FB_POST_REACTIONS_FIELD

        args = {
            'fields': enf_comma_separated("fields", fields)
        }
        resp = self._request(
            method='GET',
            path='{0}/{1}'.format(self.version, post_id),
            args=args
        )

        data = self._parse_response(resp)
        if return_json:
            return data
        else:
            return Post.new_from_json_dict(data)

    def get_posts(self,
                  ids,  # type: Optional[Union[str, List, Tuple, Set]]
                  fields=None,  # type: Optional[Union[str, List, Tuple, Set]]
                  return_json=False  # type: bool
                  ):
        # type: (...) -> dict
        """
        Retrieve multi posts info by one request.
        :param ids: Comma-separated id string for page which you want to get.
        You can also pass this with an id list, tuple, set.
        :param fields:Comma-separated id string for data fields which you want.
        You can also pass this with an id list, tuple, set.
        :param return_json: Set to false will return a dict of Post instances.
        Or return json data. Default is false.
        """
        if fields is None:
            fields = constant.FB_POST_BASIC_FIELDS + constant.FB_POST_REACTIONS_FIELD

        args = {
            "ids": enf_comma_separated("ids", ids),
            "fields": enf_comma_separated("fields", fields)
        }
        resp = self._request(
            method='GET',
            path='{0}/'.format(self.version),
            args=args
        )

        data = self._parse_response(resp)
        if return_json:
            return data
        else:
            return {_id: Post.new_from_json_dict(p_data) for _id, p_data in iteritems(data)}

    def get_comments_by_object(self,
                               object_id,  # type: str
                               summary=True,  # type: bool
                               fields=None,  # type: Optional[Union[str, List, Tuple, Set]]
                               filter_type='toplevel',  # type: str
                               order_type='chronological',  # type: str
                               count=10,  # type: Optional[int]
                               limit=50,  # type: int
                               return_json=False  # type: bool
                               ):
        # type: (...) -> (List[Union[Comment, dict]], Union[CommentSummary, dict])
        """
        Retrieve object's comments.

        Refer: https://developers.facebook.com/docs/graph-api/reference/v4.0/object/comments.

        :param object_id: The id for object(post, photo..)
        :param summary: The summary for comments
        :param fields: Comma-separated id string for data fields which you want.
        You can also pass this with an id list, tuple, set.
        :param filter_type: Valid params are toplevel/stream,
                If you chose toplevel only return top level comment.
                stream will return parent and child comment.
                default is toplevel
        :param order_type: Valid params are chronological/reverse_chronological,
                If chronological, will return comments sorted by the oldest comments first.
                If reverse_chronological, will return comments sorted by the newest comments first.
        :param count: The count will retrieve posts. If you want to get all data. Set it to None.
        :param limit: Each request retrieve posts count from api. For posts it should no more than 100.
        :param return_json: Set to false will return a list of Comment instances.
        Or return json data. Default is false.
        """
        if fields is None:
            fields = constant.FB_COMMENT_BASIC_FIELDS

        if count is not None:
            limit = min(count, limit)

        args = {
            'fields': enf_comma_separated("fields", fields),
            'summary': summary,
            'filter': filter_type,
            'order': order_type,
            'limit': limit,
        }

        comments = []
        next_cursor = None

        while True:
            next_cursor, previous_cursor, data = self.paged_by_cursor(
                resource='comments',
                target=object_id,
                args=args,
                next_cursor=next_cursor
            )
            if return_json:
                comments += data.get('data', [])
                comment_summary = data.get('summary', {})
            else:
                comments += [Comment.new_from_json_dict(item) for item in data.get('data', [])]
                comment_summary = CommentSummary.new_from_json_dict(data.get('summary', {}))
            if count is not None:
                if len(comments) >= count:
                    comments = comments[:count]
                    break
            if next_cursor is None:
                break
        return comments, comment_summary

    def get_comment_info(self,
                         comment_id,  # type: str
                         fields=None,  # type: Optional[Union[str, List, Tuple, Set]]
                         return_json=False  # type: bool
                         ):
        # type: (...) -> Union[Comment, dict]
        """
        Retrieve given comment's basic info.
        :param comment_id: The id for comment you want to retrieve data.
        :param fields: Comma-separated id string for data fields which you want.
        You can also pass this with an id list, tuple, set.
        :param return_json: Set to false will return a list of Comment instances.
        Or return json data. Default is false.
        """
        if fields is None:
            fields = constant.FB_COMMENT_BASIC_FIELDS

        args = {
            'fields': enf_comma_separated("fields", fields)
        }

        resp = self._request(
            method='GET',
            path='{0}/{1}'.format(self.version, comment_id),
            args=args
        )

        data = self._parse_response(resp)
        if return_json:
            return data
        else:
            return Comment.new_from_json_dict(data)

    def get_comments(self,
                     ids,  # type: Optional[Union[str, List, Tuple, Set]]
                     fields=None,  # type: Optional[Union[str, List, Tuple, Set]]
                     return_json=False  # type: bool
                     ):
        # type: (...) -> dict
        """
        Retrieve multi comments info by one request.
        :param ids: Comma-separated id string for page which you want to get.
        You can also pass this with an id list, tuple, set.
        :param fields:Comma-separated id string for data fields which you want.
        You can also pass this with an id list, tuple, set.
        :param return_json: Set to false will return a dict of Comment instances.
        Or return json data. Default is false.
        """
        if fields is None:
            fields = constant.FB_COMMENT_BASIC_FIELDS + constant.FB_POST_REACTIONS_FIELD

        args = {
            "ids": enf_comma_separated("ids", ids),
            "fields": enf_comma_separated("fields", fields)
        }
        resp = self._request(
            method='GET',
            path='{0}/'.format(self.version),
            args=args
        )

        data = self._parse_response(resp)
        if return_json:
            return data
        else:
            return {_id: Comment.new_from_json_dict(p_data) for _id, p_data in iteritems(data)}

    def get_picture(self,
                    page_id,  # type: str
                    pic_type=None,  # type: Optional[str]
                    return_json=False  # type: bool
                    ):
        # type: (...) -> Union[ProfilePictureSource, dict]
        """
        Retrieve the page's picture.

        :param page_id: The id for picture you want to retrieve data.
        :param pic_type: The picture type.
        :param return_json: Set to false will return a dict of Picture instances.
        Or return json data. Default is false.
        """

        if pic_type is not None and pic_type not in constant.FB_PAGE_PICTURE_TYPE:
            raise PyFacebookException(ErrorMessage(
                code=ErrorCode.INVALID_PARAMS,
                message="For field picture: pic_type must be one of the following values: {}".format(
                    ', '.join(constant.FB_PAGE_PICTURE_TYPE)
                )))

        args = {
            'redirect': 0,  # if set 0 the api will return json response.
            'type': 'normal' if pic_type is None else pic_type,
        }

        resp = self._request(
            method='GET',
            path='{0}/{1}/picture'.format(self.version, page_id),
            args=args
        )

        data = self._parse_response(resp)
        if return_json:
            return data['data']
        else:
            return ProfilePictureSource.new_from_json_dict(data['data'])

    def get_pictures(self,
                     ids,  # type: Optional[Union[str, List, Tuple, Set]]
                     pic_type=None,  # type: Optional[str]
                     return_json=False  # type: bool
                     ):
        # type: (...) -> dict
        """
        :param ids: Comma-separated id string for page which you want to get.
        You can also pass this with an id list, tuple, set.
        :param pic_type: The picture type.
        :param return_json: Set to false will return a dict of Picture instances.
        Or return json data. Default is false.
        """
        if pic_type is not None and pic_type not in constant.FB_PAGE_PICTURE_TYPE:
            raise PyFacebookException(ErrorMessage(
                code=ErrorCode.INVALID_PARAMS,
                message="For field picture: pic_type must be one of the following values: {}".format(
                    ', '.join(constant.FB_PAGE_PICTURE_TYPE)
                )))

        args = {
            "ids": enf_comma_separated("ids", ids),
            'redirect': 0,  # if set 0 the api will return json response.
            'type': 'normal' if pic_type is None else pic_type,
        }
        resp = self._request(
            method='GET',
            path='{0}/picture'.format(self.version),
            args=args
        )

        data = self._parse_response(resp)

        res = {}
        for _id, p_data in iteritems(data):
            picture_data = p_data["data"]
            if return_json:
                res[_id] = picture_data
            else:
                res[_id] = ProfilePictureSource.new_from_json_dict(picture_data)
        return res

    def get_videos_by_object(self,
                             object_id,
                             fields=None,  # type: Optional[Union[str, List, Tuple, Set]]
                             filter_type="uploaded",  # type: Optional[str]
                             count=10,  # type: Optional[int]
                             limit=25,  # type: int
                             return_json=False  # type: bool
                             ):
        # type: (...) -> List[Union[Video, dict]]
        """
        Retrieve videos from object(such as page,user,group...)
        :param object_id: The id for object(post, photo..)
        :param fields: Comma-separated id string for data fields which you want.
        You can also pass this with an id list, tuple, set.
        :param filter_type: The video type to query.
                valid parameters are:
                - uploaded
                - tagged
        :param count: The count will retrieve videos. If you want to get all data. Set it to None.
        :param limit: Each request retrieve posts count from api. For videos it should no more than 100.
        :param return_json: Set to false will return a list of Video instances.
        Or return json data. Default is false.
        :return: Videos list.
        """
        if fields is None:
            fields = constant.FB_VIDEO_BASIC_FIELDS

        if count is not None:
            limit = min(count, limit)

        args = {
            'fields': enf_comma_separated("fields", fields),
            'type': filter_type,
            'limit': limit,
        }

        videos = []
        next_cursor = None

        while True:
            next_cursor, previous_cursor, data = self.paged_by_cursor(
                resource='videos',
                target=object_id,
                args=args,
                next_cursor=next_cursor
            )
            if return_json:
                videos += data.get('data', [])
            else:
                videos += [Video.new_from_json_dict(item) for item in data.get('data', [])]
            if count is not None:
                if len(videos) >= count:
                    videos = videos[:count]
                    break
            if next_cursor is None:
                break
        return videos

    def get_video_info(self,
                       video_id,  # type: str
                       fields=None,  # type: Optional[Union[str, List, Tuple, Set]]
                       return_json=False  # type: bool
                       ):
        # type: (...) -> Union[Video, dict]
        """
        Retrieve video info by id.
        :param video_id: The id for video you want to retrieve data.
        :param fields: Comma-separated id string for data fields which you want.
        You can also pass this with an id list, tuple, set.
        :param return_json: Set to false will return a list of Video instances.
        Or return json data. Default is false.
        :return: Video instance or dict
        """
        if fields is None:
            fields = constant.FB_VIDEO_BASIC_FIELDS

        args = {
            "fields": enf_comma_separated("fields", fields)
        }

        resp = self._request(
            method='GET',
            path='{0}/{1}'.format(self.version, video_id),
            args=args
        )

        data = self._parse_response(resp)
        if return_json:
            return data
        else:
            return Video.new_from_json_dict(data)

    def get_videos(self,
                   ids,  # type: Optional[Union[str, List, Tuple, Set]]
                   fields=None,  # type: Optional[Union[str, List, Tuple, Set]]
                   return_json=False  # type: bool
                   ):
        # type: (...) -> dict
        """
        Retrieve multi videos info by one request.
        :param ids: Comma-separated id string for video which you want to get.
        You can also pass this with an id list, tuple, set.
        Notice not more than 50.
        :param fields:Comma-separated id string for data fields which you want.
        You can also pass this with an id list, tuple, set.
        :param return_json: Set to false will return a dict of Video instances.
        Or return json data. Default is false.
        :return: Videos dict.
        """
        if fields is None:
            fields = constant.FB_VIDEO_BASIC_FIELDS

        args = {
            "ids": enf_comma_separated("ids", ids),
            "fields": enf_comma_separated("fields", fields)
        }
        resp = self._request(
            method='GET',
            path='{0}/'.format(self.version),
            args=args
        )

        data = self._parse_response(resp)
        if return_json:
            return data
        else:
            return {_id: Video.new_from_json_dict(p_data) for _id, p_data in iteritems(data)}

    def get_caption_by_video_id(self,
                                video_id,  # type: str
                                fields=None,  # type: Optional[Union[str, List, Tuple, Set]]
                                return_json=False  # type: bool
                                ):
        # type: (...) -> Union[List[VideoCaption], dict]
        """
        Retrieve caption for video.
        :param video_id: The id for video caption you want to retrieve data.
        :param fields: Comma-separated id string for data fields which you want.
        You can also pass this with an id list, tuple, set.
        :param return_json: Set to false will return a list of VideoCaptions instances.
        Or return json data. Default is false.
        :return: VideoCaption instance or dict
        """
        if fields is None:
            fields = constant.FB_VIDEO_CAPTION_BASIC_FIELDS

        args = {
            "fields": enf_comma_separated("fields", fields)
        }

        resp = self._request(
            method='GET',
            path='{0}/{1}/captions'.format(self.version, video_id),
            args=args
        )

        data = self._parse_response(resp)
        if return_json:
            return data
        else:
            return [VideoCaption.new_from_json_dict(item) for item in data.get('data', [])]

    def get_caption_by_video_ids(self,
                                 ids,  # type: Optional[Union[str, List, Tuple, Set]]
                                 fields=None,  # type: Optional[Union[str, List, Tuple, Set]]
                                 return_json=False  # type: bool
                                 ):
        # type: (...) -> dict
        """
        Retrieves captions for multi videos.
        :param ids: Comma-separated id string for video captions which you want to get.
        You can also pass this with an id list, tuple, set.
        Notice not more than 50.
        :param fields:Comma-separated id string for data fields which you want.
        You can also pass this with an id list, tuple, set.
        :param return_json: Set to false will return a dict of VideoCaption instances.
        Or return json data. Default is false.
        :return: VideoCaptions dict.
        """
        if fields is None:
            fields = constant.FB_VIDEO_CAPTION_BASIC_FIELDS

        args = {
            "ids": enf_comma_separated("ids", ids),
            "fields": enf_comma_separated("fields", fields)
        }
        resp = self._request(
            method='GET',
            path='{0}/captions'.format(self.version),
            args=args
        )

        data = self._parse_response(resp)
        if return_json:
            return data
        else:
            return {_id: [Video.new_from_json_dict(item) for item in p_data["data"]] for _id, p_data in iteritems(data)}

    def get_albums_by_object(self,
                             object_id,
                             fields=None,  # type: Optional[Union[str, List, Tuple, Set]]
                             count=10,  # type: Optional[int]
                             limit=25,  # type: int
                             return_json=False  # type: bool
                             ):
        # type: (...) -> List[Union[Album, dict]]
        """
        Retrieve a list of Albums on one object.
        :param object_id: The id for object(page..)
        :param fields: Comma-separated id string for data fields which you want.
        You can also pass this with an id list, tuple, set.
        :param count: The count will retrieve videos. If you want to get all data. Set it to None.
        :param limit: Each request retrieve posts count from api. It should no more than 100.
        :param return_json: Set to false will return a list of Album instances.
        Or return json data. Default is false.
        :return: Albums list.
        """

        if fields is None:
            fields = constant.FB_ALBUM_BASIC_FIELDS

        if count is not None:
            limit = min(count, limit)

        args = {
            'fields': enf_comma_separated("fields", fields),
            'limit': limit,
        }

        albums = []
        next_cursor = None

        while True:
            next_cursor, previous_cursor, data = self.paged_by_cursor(
                resource='albums',
                target=object_id,
                args=args,
                next_cursor=next_cursor
            )
            if return_json:
                albums += data.get('data', [])
            else:
                albums += [Album.new_from_json_dict(item) for item in data.get('data', [])]
            if count is not None:
                if len(albums) >= count:
                    albums = albums[:count]
                    break
            if next_cursor is None:
                break
        return albums

    def get_album_info(self,
                       album_id,  # type: str
                       fields=None,  # type: Optional[Union[str, List, Tuple, Set]]
                       return_json=False  # type: bool
                       ):
        # type: (...) -> Union[Album, dict]
        """
        Retrieve album info by id.
        :param album_id: The id for Album you want to retrieve data.
        :param fields: Comma-separated id string for data fields which you want.
        You can also pass this with an id list, tuple, set.
        :param return_json: Set to false will return a list of Album instances.
        Or return json data. Default is false.
        :return: Album instance or dict
        """
        if fields is None:
            fields = constant.FB_ALBUM_BASIC_FIELDS

        args = {
            "fields": enf_comma_separated("fields", fields)
        }

        resp = self._request(
            method='GET',
            path='{0}/{1}'.format(self.version, album_id),
            args=args
        )

        data = self._parse_response(resp)
        if return_json:
            return data
        else:
            return Album.new_from_json_dict(data)

    def get_albums(self,
                   ids,  # type: Optional[Union[str, List, Tuple, Set]]
                   fields=None,  # type: Optional[Union[str, List, Tuple, Set]]
                   return_json=False  # type: bool
                   ):
        # type: (...) -> dict
        """
        Retrieve multi albums info by one request.
        :param ids: Comma-separated id string for albums which you want to get.
        You can also pass this with an id list, tuple, set.
        Notice not more than 50.
        :param fields:Comma-separated id string for data fields which you want.
        You can also pass this with an id list, tuple, set.
        :param return_json: Set to false will return a dict of albums instances.
        Or return json data. Default is false.
        :return: Albums dict.
        """
        if fields is None:
            fields = constant.FB_PHOTO_BASIC_FIELDS

        args = {
            "ids": enf_comma_separated("ids", ids),
            "fields": enf_comma_separated("fields", fields)
        }
        resp = self._request(
            method='GET',
            path='{0}/'.format(self.version),
            args=args
        )

        data = self._parse_response(resp)
        if return_json:
            return data
        else:
            return {_id: Album.new_from_json_dict(p_data) for _id, p_data in iteritems(data)}

    def get_photos_by_object(self,
                             object_id,
                             fields=None,  # type: Optional[Union[str, List, Tuple, Set]]
                             count=10,  # type: Optional[int]
                             limit=25,  # type: int
                             return_json=False  # type: bool
                             ):
        # type: (...) -> List[Union[Photo, dict]]
        """
        Retrieve a list of Photos on one object.
        :param object_id: The id for object(page, album..)
        :param fields: Comma-separated id string for data fields which you want.
        You can also pass this with an id list, tuple, set.
        :param count: The count will retrieve photos. If you want to get all data. Set it to None.
        :param limit: Each request retrieve posts count from api. It should no more than 100.
        :param return_json: Set to false will return a list of Photo instances.
        Or return json data. Default is false.
        :return: Photos list.
        """
        if fields is None:
            fields = constant.FB_PHOTO_BASIC_FIELDS

        if count is not None:
            limit = min(count, limit)

        args = {
            'fields': enf_comma_separated("fields", fields),
            'limit': limit,
        }

        photos = []
        next_cursor = None

        while True:
            next_cursor, previous_cursor, data = self.paged_by_cursor(
                resource='photos',
                target=object_id,
                args=args,
                next_cursor=next_cursor
            )
            if return_json:
                photos += data.get('data', [])
            else:
                photos += [Photo.new_from_json_dict(item) for item in data.get('data', [])]
            if count is not None:
                if len(photos) >= count:
                    photos = photos[:count]
                    break
            if next_cursor is None:
                break
        return photos

    def get_photo_info(self,
                       photo_id,  # type: str
                       fields=None,  # type: Optional[Union[str, List, Tuple, Set]]
                       return_json=False  # type: bool
                       ):
        # type: (...) -> Union[Photo, dict]
        """
        Retrieve photo info by id.
        :param photo_id: The id for photo you want to retrieve data.
        :param fields: Comma-separated id string for data fields which you want.
        You can also pass this with an id list, tuple, set.
        :param return_json: Set to false will return a Photo instances.
        Or return json data. Default is false.
        :return: Photo instance or dict
        """
        if fields is None:
            fields = constant.FB_PHOTO_BASIC_FIELDS

        args = {
            "fields": enf_comma_separated("fields", fields)
        }

        resp = self._request(
            method='GET',
            path='{0}/{1}'.format(self.version, photo_id),
            args=args
        )

        data = self._parse_response(resp)
        if return_json:
            return data
        else:
            return Photo.new_from_json_dict(data)

    def get_photos(self,
                   ids,  # type: Optional[Union[str, List, Tuple, Set]]
                   fields=None,  # type: Optional[Union[str, List, Tuple, Set]]
                   return_json=False  # type: bool
                   ):
        # type: (...) -> dict
        """
        Retrieve multi photos info by one request.
        :param ids: Comma-separated id string for photo which you want to get.
        You can also pass this with an id list, tuple, set.
        Notice not more than 50.
        :param fields: Comma-separated id string for data fields which you want.
        You can also pass this with an id list, tuple, set.
        :param return_json: Set to false will return a dict of Photo instances.
        Or return json data. Default is false.
        :return: Photos dict.
        """

        if fields is None:
            fields = constant.FB_PHOTO_BASIC_FIELDS

        args = {
            "ids": enf_comma_separated("ids", ids),
            "fields": enf_comma_separated("fields", fields)
        }
        resp = self._request(
            method='GET',
            path='{0}/'.format(self.version),
            args=args
        )

        data = self._parse_response(resp)
        if return_json:
            return data
        else:
            return {_id: Photo.new_from_json_dict(p_data) for _id, p_data in iteritems(data)}

    def get_live_videos_by_object(self,
                                  object_id,  # type: str
                                  fields=None,  # type: Optional[Union[str, List, Tuple, Set]]
                                  count=10,  # type: Optional[int]
                                  limit=25,  # type: int
                                  return_json=False  # type: bool
                                  ):
        # type: (...) -> List[Union[LiveVideo, dict]]
        """
        :param object_id: The id for object(page,group)
        :param fields: Comma-separated id string for data fields which you want.
        You can also pass this with an id list, tuple, set.
        :param count: The count will retrieve live videos. If you want to get all data. Set it to None.
        :param limit: Each request retrieve live videos count from api. It should no more than 100.
        :param return_json: Set to false will return a list of LiveVideo instances.
        Or return json data. Default is false.
        :return: LiveVideo list
        """
        if fields is None:
            fields = constant.FB_LIVE_VIDEO_BASIC_FIELDS

        if count is not None:
            limit = min(count, limit)

        args = {
            "fields": enf_comma_separated("fields", fields),
            "limit": limit,
        }

        live_videos = []
        next_cursor = None

        while True:
            next_cursor, previous_cursor, data = self.paged_by_cursor(
                resource='live_videos',
                target=object_id,
                args=args,
                next_cursor=next_cursor
            )
            arr = data.get("data", [])
            if return_json:
                live_videos += arr
            else:
                live_videos += [LiveVideo.new_from_json_dict(item) for item in arr]
            if count is not None:
                if len(live_videos) >= count:
                    live_videos = live_videos[:count]
                    break
            if next_cursor is None:
                break
        return live_videos

    def get_live_video_info(self,
                            live_video_id,  # type: str
                            fields=None,  # type: Optional[Union[str, List, Tuple, Set]]
                            return_json=False  # type: bool
                            ):
        # type: (...) -> Union[LiveVideo, dict]
        """
        Retrieve live video info by id.
        :param live_video_id: The id for live video.
        :param fields: Comma-separated id string for data fields which you want.
        You can also pass this with an id list, tuple, set.
        :param return_json: Set to false will return a list of LiveVideo instances.
        Or return json data. Default is false.
        :return: LiveVideo instance or dict
        """
        if fields is None:
            fields = constant.FB_LIVE_VIDEO_BASIC_FIELDS

        args = {
            "fields": enf_comma_separated("fields", fields)
        }
        resp = self._request(
            method='GET',
            path='{0}/{1}'.format(self.version, live_video_id),
            args=args
        )
        data = self._parse_response(resp)
        if return_json:
            return data
        else:
            return LiveVideo.new_from_json_dict(data)

    def get_live_videos(self,
                        ids,  # type: Optional[Union[str, List, Tuple, Set]]
                        fields=None,  # type: Optional[Union[str, List, Tuple, Set]]
                        return_json=False  # type: bool
                        ):
        # type: (...) -> dict
        """
        Retrieve multi live videos info by one request.
        :param ids: Comma-separated id string for live videos which you want to get.
        You can also pass this with an id list, tuple, set.
        Notice not more than 50.
        :param fields: Comma-separated id string for data fields which you want.
        You can also pass this with an id list, tuple, set.
        :param return_json: Set to false will return a dict of Photo instances.
        Or return json data. Default is false.
        :return: Live videos dict.
        """
        if fields is None:
            fields = constant.FB_LIVE_VIDEO_BASIC_FIELDS

        args = {
            "ids": enf_comma_separated("ids", ids),
            "fields": enf_comma_separated("fields", fields)
        }
        resp = self._request(
            method='GET',
            path='{0}/'.format(self.version),
            args=args
        )

        data = self._parse_response(resp)
        if return_json:
            return data
        else:
            return {_id: LiveVideo.new_from_json_dict(p_data) for _id, p_data in iteritems(data)}

    def get_live_video_input_stream(self,
                                    live_video_input_stream_id,  # type: str
                                    fields=None,  # type: Optional[Union[str, List, Tuple, Set]]
                                    return_json=False  # type: bool
                                    ):
        # type: (...) -> Optional[LiveVideoInputStream]
        """
        Retrieve a live video broadcast ingest stream.

        :param live_video_input_stream_id: The id for stream.
        :param fields: Comma-separated id string for data fields which you want.
        You can also pass this with an id list, tuple, set.
        :param return_json: Set to false will return a list of LiveVideoInputStream instances.
        Or return json data. Default is false.
        :return: LiveVideoInputStream instance or dict
        """
        if fields is None:
            fields = constant.FB_LIVE_VIDEO_INPUT_STREAM_BASIC_FIELDS

        args = {
            "fields": enf_comma_separated("fields", fields)
        }
        resp = self._request(
            method="GET",
            path="{0}/{1}".format(self.version, live_video_input_stream_id),
            args=args,
        )
        data = self._parse_response(resp)
        if return_json:
            return data
        else:
            return LiveVideoInputStream.new_from_json_dict(data)

    def get_live_video_input_streams(self,
                                     ids,  # type: Optional[Union[str, List, Tuple, Set]]
                                     fields=None,  # type: Optional[Union[str, List, Tuple, Set]]
                                     return_json=False  # type: bool
                                     ):
        # type: (...) -> dict
        """
        Retrieve multi live video ingest streams info by one request.
        :param ids: Comma-separated id string for live video ingest streams which you want to get.
        You can also pass this with an id list, tuple, set.
        Notice not more than 50.
        :param fields: Comma-separated id string for data fields which you want.
        You can also pass this with an id list, tuple, set.
        :param return_json: Set to false will return a dict of LiveVideoInputStream instances.
        Or return json data. Default is false.
        :return: LiveVideoInputStreams dict.
        """
        if fields is None:
            fields = constant.FB_LIVE_VIDEO_INPUT_STREAM_BASIC_FIELDS

        args = {
            "ids": enf_comma_separated("ids", ids),
            "fields": enf_comma_separated("fields", fields)
        }
        resp = self._request(
            method='GET',
            path='{0}/'.format(self.version),
            args=args
        )

        data = self._parse_response(resp)
        if return_json:
            return data
        else:
            return {_id: LiveVideoInputStream.new_from_json_dict(p_data) for _id, p_data in iteritems(data)}



import json
import unittest

import pyfacebook.models as models


class PhotoModelTest(unittest.TestCase):
    BASE_PATH = "testdata/facebook/models/photos/"

    with open(BASE_PATH + 'photo.json', 'rb') as f:
        PHOTO_INFO = json.loads(f.read().decode('utf-8'))

    def testPhoto(self):
        m = models.Photo.new_from_json_dict(self.PHOTO_INFO)

        self.assertEqual(m.id, "166370841591183")
        self.assertEqual(m.album.id, "108824087345859")
        self.assertEqual(len(m.images), 8)
        self.assertEqual(m.webp_images[0].height, 800)



import json
import unittest

import responses
from six import iteritems

import pyfacebook


class ApiRepliesTest(unittest.TestCase):
    BASE_PATH = "testdata/instagram/apidata/replies/"
    BASE_URL = "https://graph.facebook.com/{}/".format(pyfacebook.Api.VALID_API_VERSIONS[-1])

    with open(BASE_PATH + "replies_default_p1.json", "rb") as f:
        REPLIES_DEFAULT_p1 = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "replies_default_p2.json", "rb") as f:
        REPLIES_DEFAULT_p2 = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "replies_fields_p1.json", "rb") as f:
        REPLIES_FIELDS_p1 = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "replies_fields_p2.json", "rb") as f:
        REPLIES_FIELDS_p2 = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "reply_single_default.json", "rb") as f:
        REPLY_SINGLE_DEFAULT = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "reply_single_fields.json", "rb") as f:
        REPLY_SINGLE_FIELDS = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "replies_multi_default.json", "rb") as f:
        REPLIES_MULTI_DEFAULT = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "replies_multi_fields.json", "rb") as f:
        REPLIES_MULTI_FIELDS = json.loads(f.read().decode("utf-8"))

    def setUp(self):
        self.api = pyfacebook.IgProApi(
            app_id="123456", app_secret="secret",
            long_term_token="token",
        )

    def testGetRepliesByComment(self):
        comment_id = "17984127178281340"
        # test get all replies
        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL + comment_id + "/replies", json=self.REPLIES_DEFAULT_p1)
            m.add("GET", self.BASE_URL + comment_id + "/replies", json=self.REPLIES_DEFAULT_p2)

            res = self.api.get_replies_by_comment(
                comment_id=comment_id,
                count=None,
                limit=2
            )

            self.assertEqual(len(res), 3)
            self.assertEqual(res[0].id, "18107567341036926")

        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL + comment_id + "/replies", json=self.REPLIES_FIELDS_p1)

            res = self.api.get_replies_by_comment(
                comment_id=comment_id,
                fields=["id", "timestamp", "text"],
                count=1,
                limit=2,
                return_json=True
            )

            self.assertEqual(len(res), 1)
            self.assertEqual(res[0]["id"], "18107567341036926")

    def testGetReplyInfo(self):
        reply_id = "18025866040229880"

        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL + reply_id, json=self.REPLY_SINGLE_DEFAULT)
            m.add("GET", self.BASE_URL + reply_id, json=self.REPLY_SINGLE_FIELDS)

            res_default = self.api.get_reply_info(
                reply_id=reply_id
            )
            self.assertEqual(res_default.id, reply_id)

            res_fields = self.api.get_reply_info(
                reply_id=reply_id,
                fields=["id", "timestamp", "text"],
                return_json=True
            )
            self.assertEqual(res_fields["id"], reply_id)

    def testGetRepliesInfo(self):
        reply_ids = ["17850033184810160", "18025866040229880"]

        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL, json=self.REPLIES_MULTI_DEFAULT)
            m.add("GET", self.BASE_URL, json=self.REPLIES_MULTI_FIELDS)

            res_default = self.api.get_replies_info(
                reply_ids=reply_ids
            )
            for _id, data in iteritems(res_default):
                self.assertIn(_id, reply_ids)
                self.assertIn(_id, data.id)

            res_fields = self.api.get_replies_info(
                reply_ids=reply_ids,
                fields=["id", "timestamp", "text"],
                return_json=True
            )
            for _id, data in iteritems(res_fields):
                self.assertIn(_id, reply_ids)
                self.assertIn(_id, data["id"])



import json
import unittest

import responses
from six import iteritems

import pyfacebook


class LiveVideoApiTest(unittest.TestCase):
    BASE_PATH = "testdata/facebook/apidata/live/"
    BASE_URL = "https://graph.facebook.com/v8.0/"

    with open(BASE_PATH + "live_videos_p1.json", "rb") as f:
        LIVE_VIDEOS_PAGED_1 = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "live_videos_p2.json", "rb") as f:
        LIVE_VIDEOS_PAGED_2 = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "live_video.json", "rb") as f:
        LIVE_VIDEO = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "live_videos.json", "rb") as f:
        LIVE_VIDEOS = json.loads(f.read().decode("utf-8"))

    def setUp(self):
        self.api = pyfacebook.Api(
            app_id="12345678",
            app_secret="secret",
            long_term_token="token",
            version="v8.0"
        )

    def testGetLiveVideosByObject(self):
        page_id = "2121008874780932"

        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL + page_id + "/live_videos", json=self.LIVE_VIDEOS_PAGED_1)
            m.add("GET", self.BASE_URL + page_id + "/live_videos", json=self.LIVE_VIDEOS_PAGED_2)

            live_videos = self.api.get_live_videos_by_object(
                object_id=page_id,
                count=None,
                limit=2,
            )
            self.assertEqual(len(live_videos), 4)

        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL + page_id + "/live_videos", json=self.LIVE_VIDEOS_PAGED_1)
            m.add("GET", self.BASE_URL + page_id + "/live_videos", json=self.LIVE_VIDEOS_PAGED_2)

            live_videos = self.api.get_live_videos_by_object(
                object_id=page_id,
                count=3,
                limit=2,
                return_json=True
            )
            self.assertEqual(len(live_videos), 3)

    def testGetLiveVideo(self):
        vid = "2814245952123884"

        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL + vid, json=self.LIVE_VIDEO)

            live_video = self.api.get_live_video_info(
                live_video_id=vid,
                fields=["id", "title", "creation_time", "status"],
            )
            self.assertEqual(live_video.id, vid)
            self.assertEqual(live_video.status, "VOD")

            lv_json = self.api.get_live_video_info(
                live_video_id=vid,
                return_json=True
            )
            self.assertEqual(lv_json["id"], vid)

    def testGetLiveVideos(self):
        ids = ["2809188389296307", "2814245952123884"]

        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL, json=self.LIVE_VIDEOS)

            video_dict = self.api.get_live_videos(
                ids=ids,
            )
            for _id, data in iteritems(video_dict):
                self.assertIn(_id, ids)
                self.assertEqual(_id, data.id)

            video_dict = self.api.get_live_videos(
                ids=ids,
                fields=["id", "description", "content_category", "created_time"],
                return_json=True
            )
            for _id, data in iteritems(video_dict):
                self.assertIn(_id, ids)
                self.assertEqual(_id, data["id"])


class LiveVideoInputStream(unittest.TestCase):
    BASE_PATH = "testdata/facebook/apidata/live/"
    BASE_URL = "https://graph.facebook.com/v8.0/"

    with open(BASE_PATH + "stream.json", "rb") as f:
        STREAM = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "streams.json", "rb") as f:
        STREAMS = json.loads(f.read().decode("utf-8"))

    def setUp(self):
        self.api = pyfacebook.Api(
            app_id="12345678",
            app_secret="secret",
            long_term_token="token",
            version="v8.0"
        )

    def testGetInputStream(self):
        stream_id = "2814245955457217"

        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL + stream_id, json=self.STREAM)

            stream = self.api.get_live_video_input_stream(
                live_video_input_stream_id=stream_id,
                fields=["id", "dash_ingest_url", "is_master", "stream_health"],
            )

            self.assertEqual(stream.id, stream_id)
            self.assertEqual(stream.stream_health.video_bitrate, 0)

            stream_json = self.api.get_live_video_input_stream(
                live_video_input_stream_id=stream_id,
                return_json=True
            )
            self.assertEqual(stream_json["id"], stream_id)

    def testGetInputStreams(self):
        ids = ["2809188399296306", "2814245955457217"]

        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL, json=self.STREAMS)

            stream_dict = self.api.get_live_video_input_streams(
                ids=ids,
            )
            for _id, data in iteritems(stream_dict):
                self.assertIn(_id, ids)
                self.assertEqual(_id, data.id)

            stream_dict = self.api.get_live_video_input_streams(
                ids=ids,
                fields=["id", "description", "content_category", "created_time"],
                return_json=True
            )
            for _id, data in iteritems(stream_dict):
                self.assertIn(_id, ids)
                self.assertEqual(_id, data["id"])



import json
import unittest

import pyfacebook.models as models


class IgProModelTest(unittest.TestCase):
    BASE_PATH = "testdata/instagram/models/"

    with open(BASE_PATH + 'ig_user.json', 'rb') as f:
        IG_USER = json.loads(f.read().decode('utf-8'))
    with open(BASE_PATH + 'ig_media.json', 'rb') as f:
        IG_MEDIA = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + 'ig_comment.json', 'rb') as f:
        IG_COMMENT = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + 'ig_reply.json', 'rb') as f:
        IG_REPLY = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + 'ig_hashtag.json', 'rb') as f:
        IG_HASHTAG = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "ig_insight.json", "rb") as f:
        IG_INSIGHT = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "ig_story.json", "rb") as f:
        IG_STORY = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "ig_container.json", "rb") as f:
        IG_CONTAINER = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "ig_publish_limit.json", "rb") as f:
        IG_PUBLISH_LIMIT = json.loads(f.read().decode("utf-8"))

    def testUser(self):
        m = models.IgProUser.new_from_json_dict(self.IG_USER)

        self.assertEqual(m.id, "17841406338772941")

    def testMedia(self):
        m = models.IgProMedia.new_from_json_dict(self.IG_MEDIA)

        self.assertEqual(m.comments_count, 6)
        self.assertEqual(len(m.children), 3)
        self.assertEqual(len(m.comments), 3)

    def testComment(self):
        m = models.IgProComment.new_from_json_dict(self.IG_COMMENT)

        self.assertEqual(m.like_count, 0)
        self.assertEqual(m.media.id, "17955956875141196")
        self.assertEqual(len(m.replies), 3)

    def testReply(self):
        m = models.IgProReply.new_from_json_dict(self.IG_REPLY)

        self.assertEqual(m.id, "18107567341036926")
        self.assertEqual(m.user.id, "17841406338772941")

    def testHashtag(self):
        m = models.IgProHashtag.new_from_json_dict(self.IG_HASHTAG)

        self.assertEqual(m.id, "17841593698074073")

    def testInsight(self):
        m = models.IgProInsight.new_from_json_dict(self.IG_INSIGHT)

        self.assertEqual(m.name, "impressions")
        self.assertEqual(m.id, "instagram_business_account_id/insights/impressions/day")
        self.assertEqual(len(m.values), 2)
        self.assertEqual(m.values[0].value, 32)

    def testStory(self):
        m = models.IgProStory.new_from_json_dict(self.IG_STORY)

        self.assertEqual(m.id, "17908009870517752")
        self.assertEqual(m.owner.id, "17841406338772941")

    def testContainer(self):
        m = models.IgProContainer.new_from_json_dict(self.IG_CONTAINER)

        self.assertEqual(m.id, "17889615691921648")
        self.assertEqual(m.status_code, "FINISHED")

    def testPublishLimit(self):
        m = models.IgProPublishLimit.new_from_json_dict(self.IG_PUBLISH_LIMIT)

        self.assertEqual(m.quota_usage, 2)
        self.assertEqual(m.config.quota_total, 25)



"""
    models for instagram platform
"""

from attr import attrs, attrib
from typing import Dict, List, Optional, Any

from .base import BaseModel
from .._compat import str


@attrs
class IgProUser(BaseModel):
    """
    A class representing the Instagram user info.

    Refer: https://developers.facebook.com/docs/instagram-api/reference/user
    """
    biography = attrib(default=None, type=Optional[str], repr=False)
    id = attrib(default=None, type=Optional[str])
    ig_id = attrib(default=None, type=Optional[int], repr=False)
    followers_count = attrib(default=None, type=Optional[int], repr=False)
    follows_count = attrib(default=None, type=Optional[int], repr=False)
    media_count = attrib(default=None, type=Optional[int], repr=False)
    name = attrib(default=None, type=Optional[str])
    username = attrib(default=None, type=Optional[str])
    profile_picture_url = attrib(default=None, type=Optional[str], repr=False)
    website = attrib(default=None, type=Optional[str], repr=False)


@attrs
class IgProMediaChildren(BaseModel):
    """
    A class representing the Instagram media children info. This is subset for media

    Refer: https://developers.facebook.com/docs/instagram-api/reference/media
    """
    id = attrib(default=None, type=Optional[str])
    ig_id = attrib(default=None, type=Optional[int], repr=False)
    media_type = attrib(default=None, type=Optional[str], repr=False)
    media_url = attrib(default=None, type=Optional[str], repr=False)
    owner = attrib(default=None, type=Optional[IgProUser], repr=False)
    permalink = attrib(default=None, type=Optional[str])
    shortcode = attrib(default=None, type=Optional[str], repr=False)
    thumbnail_url = attrib(default=None, type=Optional[str], repr=False)
    timestamp = attrib(default=None, type=Optional[str], repr=False)
    username = attrib(default=None, type=Optional[str], repr=False)


@attrs
class IgProBaseComment(BaseModel):
    """
    A class representing the Instagram comment info. Base fields.

    Refer: https://developers.facebook.com/docs/instagram-api/reference/comment
    """
    hidden = attrib(default=None, type=Optional[bool], repr=False)
    id = attrib(default=None, type=Optional[str])
    like_count = attrib(default=None, type=Optional[int], repr=False)
    media = attrib(default=None, type=Optional[Dict], repr=False)
    text = attrib(default=None, type=Optional[str], repr=False)
    timestamp = attrib(default=None, type=Optional[str])
    user = attrib(default=None, type=Optional[IgProUser], repr=False)
    username = attrib(default=None, type=Optional[str], repr=False)

    def __attrs_post_init__(self):
        if self.media is not None and isinstance(self.media, dict):
            self.media = IgProMedia.new_from_json_dict(self.media)


@attrs
class IgProReply(IgProBaseComment):
    """
    A class representing the Instagram replay info. This is similar to comment but have no replies.

    Refer: https://developers.facebook.com/docs/instagram-api/reference/comment
    """


@attrs
class IgProComment(IgProBaseComment):
    """
    A class representing the Instagram comment info.

    Refer: https://developers.facebook.com/docs/instagram-api/reference/comment
    """
    replies = attrib(default=None, type=Optional[Dict], repr=False)

    def __attrs_post_init__(self):
        IgProBaseComment.__attrs_post_init__(self)
        if self.replies is not None and isinstance(self.replies, dict):
            replies = self.replies.get("data", [])
            self.replies = [IgProReply.new_from_json_dict(item) for item in replies]


@attrs
class MediaCommon(BaseModel):
    """
    Some common fields for media or story
    """
    id = attrib(default=None, type=Optional[str])
    ig_id = attrib(default=None, type=Optional[int], repr=False)
    caption = attrib(default=None, type=Optional[str], repr=False)
    comments_count = attrib(default=None, type=Optional[int], repr=False)
    like_count = attrib(default=None, type=Optional[int], repr=False)
    media_product_type = attrib(default=None, type=Optional[str], repr=False)
    media_type = attrib(default=None, type=Optional[str], repr=False)
    media_url = attrib(default=None, type=Optional[str], repr=False)
    owner = attrib(default=None, type=Optional[IgProUser], repr=False)
    permalink = attrib(default=None, type=Optional[str])
    shortcode = attrib(default=None, type=Optional[str], repr=False)
    thumbnail_url = attrib(default=None, type=Optional[str], repr=False)
    timestamp = attrib(default=None, type=Optional[str], repr=False)
    username = attrib(default=None, type=Optional[str], repr=False)
    video_title = attrib(default=None, type=Optional[str], repr=False)


@attrs
class IgProMedia(MediaCommon):
    """
    A class representing the Instagram media info.

    Refer: https://developers.facebook.com/docs/instagram-api/reference/media
    """
    is_comment_enabled = attrib(default=None, type=Optional[bool], repr=False)

    # connections
    comments = attrib(default=None, type=Optional[Dict], repr=False)
    children = attrib(default=None, type=Optional[Dict], repr=False)

    def __attrs_post_init__(self):
        if self.children is not None and isinstance(self.children, dict):
            children = self.children.get("data", [])
            self.children = [IgProMediaChildren.new_from_json_dict(item) for item in children]
        if self.comments is not None and isinstance(self.comments, dict):
            comments = self.comments.get("data", [])
            self.comments = [IgProComment.new_from_json_dict(item) for item in comments]


@attrs
class IgProStory(MediaCommon):
    """
    A class representing the Instagram story info. It's similar to media but not have some fields.

    Refer: https://developers.facebook.com/docs/instagram-api/reference/ig-user/stories
    """
    pass


@attrs
class IgProHashtag(BaseModel):
    """
    A class representing the Instagram hashtag info.

    Refer: https://developers.facebook.com/docs/instagram-api/reference/hashtag
    """
    id = attrib(default=None, type=Optional[str])
    name = attrib(default=None, type=Optional[str])


@attrs
class IgProInsightValue(BaseModel):
    """
    A class representing the Instagram insight value info.

    Refer: https://developers.facebook.com/docs/instagram-api/guides/insights#getting-account-metrics
    """
    value = attrib(default=None, type=Optional[Any])
    end_time = attrib(default=None, type=Optional[str])


@attrs
class IgProInsight(BaseModel):
    """
    A class representing the Instagram insight info.

    Refer: https://developers.facebook.com/docs/instagram-api/guides/insights#getting-account-metrics
    """

    id = attrib(default=None, type=Optional[str], repr=False)
    name = attrib(default=None, type=Optional[str])
    period = attrib(default=None, type=Optional[str])
    title = attrib(default=None, type=Optional[str], repr=False)
    description = attrib(default=None, type=Optional[str], repr=False)
    values = attrib(default=None, type=Optional[List[Dict]], repr=False)

    def __attrs_post_init__(self):
        if self.values is not None and isinstance(self.values, List):
            values = self.values
            self.values = [IgProInsightValue.new_from_json_dict(item) for item in values]


@attrs
class IgProContainer(BaseModel):
    """
    A class representing the Instagram container info.

    Refer: https://developers.facebook.com/docs/instagram-api/reference/ig-container
    """

    id = attrib(default=None, type=Optional[str])
    status_code = attrib(default=None, type=Optional[str])
    status = attrib(default=None, type=Optional[str], repr=False)


@attrs
class IgProPublishLimitConfig(BaseModel):
    """
    A class representing the Instagram publish limit config

    Refer: https://developers.facebook.com/docs/instagram-api/reference/ig-user/content_publishing_limit#fields
    """

    quota_total = attrib(default=None, type=Optional[int])
    quota_duration = attrib(default=None, type=Optional[int])


@attrs
class IgProPublishLimit(BaseModel):
    """
    A class representing the Instagram publish limit

    Refer: https://developers.facebook.com/docs/instagram-api/reference/ig-user/content_publishing_limit#fields
    """

    quota_usage = attrib(default=None, type=Optional[int])
    config = attrib(default=None, type=Optional[IgProPublishLimitConfig], repr=False)



import json
import unittest

import pyfacebook.models as models


class VideoModelTest(unittest.TestCase):
    BASE_PATH = "testdata/facebook/models/videos/"

    with open(BASE_PATH + 'video.json', 'rb') as f:
        VIDEO_INFO = json.loads(f.read().decode('utf-8'))
    with open(BASE_PATH + 'caption.json', 'rb') as f:
        CAPTION_INFO = json.loads(f.read().decode('utf-8'))

    def testVideo(self):
        m = models.Video.new_from_json_dict(self.VIDEO_INFO)

        self.assertEqual(m.id, "969222676905304")
        self.assertEqual(m.created_time, "2020-09-12T09:53:06+0000")
        self.assertEqual(len(m.content_tags), 2)
        self.assertEqual(m.format[0].height, 229)
        self.assertEqual(m.place.id, "406584169736960")
        self.assertEqual(m.place.location.zip, "100037")
        self.assertEqual(m.status.video_status, "ready")

        self.assertEqual(m.privacy.value, "EVERYONE")
        self.assertFalse(m.is_crosspost_video)

        self.assertEqual(m.likes.total_count, 1525)
        self.assertEqual(m.comments.total_count, 139)

    def testCaption(self):
        m = models.VideoCaption.new_from_json_dict(self.CAPTION_INFO)

        self.assertEqual(m.locale, "zh_CN")
        self.assertEqual(m.create_time, "1599928066")
